-- - prologOra.ddl start ---------------------------------------------
-- Version: 1.3
-- -------------------------------------------------------------------
-- DDL for @PRODUCT.SERVER@ @PRODUCT.VERSION.DETAIL@ - @BLDLEVEL@
--
-- - prologOra.ddl end -----------------------------------------------


-- ESB requires a view to be created in the ORACLE DATABASE.. 

------------------------------------------------
-- DDL Statements for table "MESSAGELOG"
------------------------------------------------
 
CREATE TABLE MESSAGELOG
  (GTRANID VARCHAR(127) NOT NULL,
   TIMESTAMP TIMESTAMP NOT NULL,
   MEDIATIONNAME VARCHAR(256), 
   MODULENAME VARCHAR(256), 
   VERSION VARCHAR(50),
   REQUESTERID VARCHAR(250),
   MESSAGE CLOB);

-- DDL Statements for primary key on Table "MESSAGELOG"

ALTER TABLE MESSAGELOG
  ADD CONSTRAINT MESSAGELOG_KEY
	PRIMARY KEY (GTRANID, TIMESTAMP);


-- Comments for the Table MESSAGELOG

COMMENT ON TABLE MESSAGELOG IS 'Logged SMO information';
COMMENT ON COLUMN MESSAGELOG.GTRANID IS 'The global transaction ID';
COMMENT ON COLUMN MESSAGELOG.TIMESTAMP IS 'The time at which the record was added';
COMMENT ON COLUMN MESSAGELOG.MEDIATIONNAME IS 'The mediation primitive instance name';
COMMENT ON COLUMN MESSAGELOG.MODULENAME IS 'The name of the FLOW';
COMMENT ON COLUMN MESSAGELOG.VERSION IS 'The version information in the SMO';
COMMENT ON COLUMN MESSAGELOG.REQUESTERID IS 'Requester ID';
COMMENT ON COLUMN MESSAGELOG.MESSAGE IS 'The part of the SMO which has been logged';

------------------------------------------------
-- DDL Statements for table "TARGETLOG"
------------------------------------------------
 
CREATE TABLE TARGETLOG
  (GTRANID VARCHAR(127) NOT NULL,
   TARGETID VARCHAR(250) NOT NULL, 
   TIMESTAMP TIMESTAMP NOT NULL);

-- DDL Statements for primary key on Table "TARGETLOG"

ALTER TABLE TARGETLOG
  ADD CONSTRAINT TARGETLOG_KEY
	PRIMARY KEY (GTRANID, TARGETID, TIMESTAMP);

-- DDL Statements for foreign key on Table "TARGETLOG"

ALTER TABLE TARGETLOG
  ADD CONSTRAINT TARGETLOG_MESSAGELOG_KEY
	FOREIGN KEY (GTRANID, TIMESTAMP)
    		REFERENCES MESSAGELOG (GTRANID, TIMESTAMP)
    		ON DELETE CASCADE;

-- Comments for the Table TARGETLOG

COMMENT ON TABLE TARGETLOG IS 'The tracked target address information';
COMMENT ON COLUMN TARGETLOG.GTRANID IS 'The global transaction ID';
COMMENT ON COLUMN TARGETLOG.TARGETID IS 'The target address';
COMMENT ON COLUMN TARGETLOG.TIMESTAMP IS 'The time at which the record was added';


------------------------------------------------
-- DDL Statements for table "SLADATA"
------------------------------------------------

-- create the sequence for SLAData table
create sequence SLA_KEY_SEQ;

-- create the SLA table
create table SLADATA (SEQNO number(38,0) not null primary key, REQUESTOR varchar(256) not null, SERVICE varchar(256) not null, OPERATION varchar(256) not null, CLEANUP number(38,0) not null, USEDTOKENS number(38,0));

-- create an index
create index I1 on SLADATA (CLEANUP, REQUESTOR, SERVICE, OPERATION);

-- Comments for the Table
COMMENT ON TABLE SLADATA IS 'SLA runtime data';
COMMENT ON COLUMN SLADATA.SEQNO IS 'Unique sequence number';
COMMENT ON COLUMN SLADATA.REQUESTOR IS 'Requester identity';
COMMENT ON COLUMN SLADATA.SERVICE IS 'Service being invoked';
COMMENT ON COLUMN SLADATA.OPERATION IS 'Service operation being invoked';
COMMENT ON COLUMN SLADATA.CLEANUP IS 'Clean up interval';
COMMENT ON COLUMN SLADATA.USEDTOKENS IS 'Used tokens';


COMMIT;