#   begin_generated_IBM_prolog
#
#   prolog.pro.samp.telcosp
#   ******************************************************************
#   IBM Sample Source Materials
#
#   Sample source materials are supplied As-Is.
#   No warranty is expressed or implied.
#
#   Product(s): 5724-O05 soa
#
#   (C)Copyright IBM Corp. 2008
#
#   The source code for this program supplied under the terms of the
#   End User License Agreement (EULA) that accompanied this product.
#   ******************************************************************
#
#   end_generated_IBM_prolog

import time, os, sys, os.path, shutil
from java.lang import System, Runtime, String, Long, Float, Integer, Exception
from java.util import Properties
from java.io import File, FileInputStream
from pytwss.utils import _, setLogFile, log, logScreen
import initdb
import wsadmin_utils_cfg, wsadmin_utils_cfg_db

COMPONENT_NAME = "common_spm"

#
# Perform the configuration process.
# param: java.util.Properties response_properties object with the defined properties
# param: deploy_info object used to query the server or cluster members.
# return: the result "SUCCESS" "FAILURE"
#
def configure(response_properties, deploy_info, selectedFeatures_properties):
    
    # get variables
    ddl_root = response_properties.get("DDL_PATH")
    ddl_type = response_properties.get("DDL_TYPE")
    was_home = response_properties.get("WAS_HOME")
    component_marker_path = response_properties.getProperty("COMPONENT_MARKER_PATH")
        
    # check component marker
    #spm_rt_marker_path = component_marker_path + os.sep + "spm_runtime.properties"
    #spm_cons_marker_path = component_marker_path + os.sep + "spm_console.properties"
    
    spm_rt_marker_path = selectedFeatures_properties.getProperty("SPM_Runtime")
    spm_cons_marker_path = selectedFeatures_properties.getProperty("SPM_Console")
    
    # require that a SPM component is installed to do SPM configuration.
    #if(os.path.exists(spm_rt_marker_path) != 1 and os.path.exists(spm_cons_marker_path) != 1):
    if spm_rt_marker_path == 'false' and spm_cons_marker_path == 'false':
       log("Skip Configuring: " + COMPONENT_NAME)
       return "SUCCESS"
           	
    log("Configuring: " + COMPONENT_NAME)
    logScreen(_("SOAC7533I") %{ '1' : COMPONENT_NAME })
    
    # do initialization
    #
    #  Note: the SPM database tables are intentionally initialized from common_twss, rather than here so that we can initialize policies without installing
    #  SPM_runtime on this cluster.  It may be installed elsewhere.
    #

    # data source provider
    scope = deploy_info.getClusterId()
    if scope == None or scope == "":
       scope = deploy_info.getCellId()
    dbType = response_properties.get("DatabaseServerType")
    provider = wsadmin_utils_cfg_db.createJDBCProvider(dbType, "${ORACLE_JDBC_DRIVER_PATH}/ojdbc14.jar", "", "false", scope)
    
    # data source
    dbName = response_properties.get("DatabaseName")
    dbServer = response_properties.get("DatabaseServerHostname")
    dbPort = response_properties.get("DatabaseServerPort")
    dbUserUserName =  response_properties.get("DatabaseServerUserName")
    dbUserPassword =  response_properties.get("DatabaseServerUserPassword")

    wsadmin_utils_cfg.createJAASAuthData("SPMAlias",  dbUserUserName, dbUserPassword, "database and queue client identity", scope)
    dsSPM = wsadmin_utils_cfg_db.createGenericDatasource(dbType, "SPMDataSource", "jdbc/SPMDB", 128, provider, "SPMAlias", "SPM DataSource", dbName, scope, dbServer, dbPort)

    return "SUCCESS"
    
#
# Perform the syncServers process
# param: java.util.Properties response_properties object with the defined properties 
# return: the result "SUCCESS" "FAILURE"
#
def syncServers(response_properties, deploy_info, selectedFeatures_properties):

    # get variables
    was_home = response_properties.get("WAS_HOME")
    component_marker_path = response_properties.getProperty("COMPONENT_MARKER_PATH")

    # check component marker
    #spm_rt_marker_path = component_marker_path + os.sep + "spm_runtime.properties"
    #spm_cons_marker_path = component_marker_path + os.sep + "spm_console.properties"
    spm_rt_marker_path = selectedFeatures_properties.getProperty("SPM_Runtime")
    spm_cons_marker_path = selectedFeatures_properties.getProperty("SPM_Console")
    
    # require that a SPM component is installed to do SPM configuration.
    #if(os.path.exists(spm_rt_marker_path) != 1 and os.path.exists(spm_cons_marker_path) != 1):
    if spm_rt_marker_path == 'false' and spm_cons_marker_path == 'false':
       log("Skip SyncServers: " + COMPONENT_NAME)
       return "SUCCESS"

    log("SyncServers: " + COMPONENT_NAME)
    logScreen(_("SOAC7533I") %{ '1' : COMPONENT_NAME })
    
    # do initialization
    # more later 

    return "SUCCESS"
    