#   begin_generated_IBM_prolog
#
#   prolog.pro.samp.telcosp
#   ******************************************************************
#   IBM Sample Source Materials
#
#   Sample source materials are supplied As-Is.
#   No warranty is expressed or implied.
#
#   Product(s): 5724-O05 soa
#
#   (C)Copyright IBM Corp. 2008
#
#   The source code for this program supplied under the terms of the
#   End User License Agreement (EULA) that accompanied this product.
#   ******************************************************************
#
#   end_generated_IBM_prolog

import time, os, sys, os.path, shutil
from java.lang import System, Runtime, String, Long, Float, Integer, Exception
from java.util import Properties
from java.io import File, FileInputStream
from component_ag_runtime import COMPONENT_NAME as AG_RT_COMP_NAME
from wsadmin_utils import getOsPath
from pytwss.mbean.file_transfer_mbean import downloadFile
from pytwss.utils import _, setLogFile, log, logScreen
import initdb
import wsadmin_utils, wsadmin_utils_cfg, wsadmin_utils_cfg_db, wsadmin_utils_cfg_jms, wsadmin_utils_cfg_app

COMPONENT_NAME = "common_svcs"

#
# Perform the configuration process.
# param: java.util.Properties response_properties object with the defined properties
# param: deploy_info object used to query the server or cluster members.
# return: the result "SUCCESS" "FAILURE"
#
def configure(response_properties, deploy_info, selectedFeatures_properties):
    
    # get variables
    was_home = response_properties.get("WAS_HOME")
    ddl_root = response_properties.get("DDL_SERVICES_PATH")
    ddl_type = response_properties.get("DDL_TYPE")
    component_marker_path = response_properties.getProperty("COMPONENT_MARKER_PATH")
    service_marker_path = response_properties.getProperty("SERVICE_MARKER_PATH")
    
    # require that the services installer has been run
    if os.path.exists(service_marker_path) != 1:
       log("Skip Configuring: " + COMPONENT_NAME)
       return "SUCCESS"
       
    log("Configuring: " + COMPONENT_NAME)
    logScreen(_("SOAC7533I") %{ '1' : COMPONENT_NAME })
    
    #if (os.path.exists(service_marker_path + os.sep + 'PX21_PRS_IMS.properties') or 
          #os.path.exists(service_marker_path + os.sep + 'PX21_TS_IMS.properties')):

    svc_prs_ims_marker_path = selectedFeatures_properties.getProperty("Presence_over_SIP")
    svc_ts_ims_marker_path = selectedFeatures_properties.getProperty("TS_over_SIP")
    
          
    if (svc_prs_ims_marker_path == 'true' or svc_ts_ims_marker_path == 'true'):          
       # Note: this AdminCmn*.ddl is only used when the presence server component is used.  Therefore not parlay.   
       ddl_path = initdb.buildDDLPath(ddl_root, "AdminCmn", ddl_type)
       if ddl_path == "FAILURE":
          return "FAILURE"
    
       # initialize database
       result = initdb.executeDDL(response_properties, ddl_path)        
       if result != "SUCCESS":
          return "FAILURE"
       
    return "SUCCESS"
    
#
# Perform the syncServers process
# param: java.util.Properties response_properties object with the defined properties 
# return: the result "SUCCESS" "FAILURE"
#
def syncServers(response_properties, deploy_info, selectedFeatures_properties):

    # get variables
    was_home = response_properties.get("WAS_HOME")
    profile_name = response_properties.get("PROFILE_NAME")	
    component_marker_path = response_properties.getProperty("COMPONENT_MARKER_PATH")
    service_marker_path = response_properties.getProperty("SERVICE_MARKER_PATH")
    
    # require that the services installer has been run
    if os.path.exists(service_marker_path) != 1:
       log("Skip SyncServers: " + COMPONENT_NAME)
       return "SUCCESS"
       
    log("SyncServers: " + COMPONENT_NAME)
    logScreen(_("SOAC7533I") %{ '1' : COMPONENT_NAME })
    
    # do initialization
    dm_host = None
    dm_info = deploy_info.getDmInfo()
    if(dm_info):
    	dm_node = dm_info.getNodeName()
    	dm_node_id = wsadmin_utils.getNodeId(dm_node)
    	dm_host = wsadmin_utils.getShortHostName(dm_node_id)
    	log("The dm host is %s." % dm_host)
		
    servers = deploy_info.getServerIds()
    for server in servers:
       wsadmin_utils_cfg.createServerThreadPool(server, "TWSSJCA_TP", 10, 50, 60000)
       wsadmin_utils_cfg.setServerThreadPoolSize("TWSSJCA_TP", 50, 80, server)
       sip_tp_id = wsadmin_utils_cfg.createServerThreadPool(server, "TWSSSIPContainer_TP", 10, 50, 60000)
       wsadmin_utils_cfg.setServerThreadPoolSize("TWSSSIPContainer_TP", 50, 80, server)
       wsadmin_utils_cfg.setSIPContainerThreadPool(sip_tp_id, server)
 
    nodes = deploy_info.getNodeNames()
    active_profile_path = was_home + os.sep + "profiles" + os.sep + profile_name
    active_profile_temp_path = active_profile_path + os.sep + "config" + os.sep + "temp"
			 
    for node in nodes:
       log("Node: %s" %(node))
       scope = wsadmin_utils.getNodeId(node)
       node_id = wsadmin_utils.getNodeId(node)
       node_host = wsadmin_utils.getShortHostName(node_id)
       
       # install the RAR files here, since they might be common to multiple components
       was_node_home = wsadmin_utils_cfg.getEnvironmentVariable("WAS_INSTALL_ROOT", node_id)
       log("The node's was_home is %s" % was_node_home)

       smssmpp_marker_file_path = service_marker_path + File.separator + "PX21_SMS_SMPP.properties"
       wapsmpp_marker_file_path = service_marker_path + File.separator + "WAP10_PUSH_SMPP.properties"  

       smssmpp_marker_path = selectedFeatures_properties.getProperty("SMS_over_SMPP")
       wapsmpp_marker_path = selectedFeatures_properties.getProperty("WAP_Push_over_SMPP")
       
       smpp_rar_short_path = "installableApps" + File.separator + "TWSS-Services" + File.separator + "SMPPConnector.rar"
       smpp_rar_path = was_home + File.separator + smpp_rar_short_path
       smpp_node_rar_path = was_node_home + File.separator + smpp_rar_short_path
       smpp_temp_path = active_profile_temp_path + os.sep + smpp_rar_short_path[:smpp_rar_short_path.rindex(os.sep)]
	   
       #if os.path.exists(smssmpp_marker_path) == 1 or os.path.exists(wapsmpp_marker_path) == 1:
       if smssmpp_marker_path == 'true' or wapsmpp_marker_path == 'true':
          # install SMPP RAR
          if os.path.exists(smpp_rar_path) == 1:
		  	         
             if((dm_host and dm_host == node_host) or not dm_host):
                log("Skipping file transfer to: %s" % dm_host)
                logScreen(_("SOAC7539I") %{ '1' : node })
             else:   
                if(os.path.exists(smpp_temp_path) != 1):
                   log("Making directory %s. " % smpp_temp_path)
                   os.makedirs(smpp_temp_path)
                shutil.copyfile(smpp_rar_path, active_profile_temp_path + os.sep + smpp_rar_short_path)
                log("Publishing SMPP Rar file: %s to node: %s." % (smpp_rar_short_path, node))
                downloadFile(smpp_rar_short_path , File(".." + os.sep + ".." + os.sep + smpp_rar_short_path), node, 0)
                log("Publish successful.")
             res_id = wsadmin_utils_cfg_app.installResourceAdapter(smpp_node_rar_path, node, "SMPPConnector", "\"For SMS/SMPP and WAP/SMPP\"", "TWSSJCA_TP")
             
             if os.path.exists(smssmpp_marker_file_path) == 1:
                wsadmin_utils_cfg_app.createJ2CConnectionFactory(res_id, "PX21_SMS_SMPPJ2CCF", "eis/PX21_SMS_SMPPJ2CCF", "TWSSAlias", scope)
                wsadmin_utils_cfg_app.createJ2CActivationSpec(res_id, "PX21_SMS_SMPPCallbackJ2CAS", "eis/PX21_SMS_SMPPCallbackJ2CAS", "TWSSAlias", scope)
                
             if os.path.exists(wapsmpp_marker_file_path) == 1:
                wsadmin_utils_cfg_app.createJ2CConnectionFactory(res_id, "WAP10_Push_SMPPJ2CCF", "eis/WAP10_Push_SMPPJ2CCF", "TWSSAlias", scope)
                wsadmin_utils_cfg_app.createJ2CActivationSpec(res_id, "WAP10_Push_SMPPCallbackJ2CAS", "eis/WAP10_Push_SMPPCallbackJ2CAS", "TWSSAlias", scope)

          else:
             log("ERROR the rar path does not exist: %s" %(smpp_rar_path))
             return "FAILURE"
                   
    return "SUCCESS"
    
