#   begin_generated_IBM_prolog
#
#   prolog.pro.samp.telcosp
#   ******************************************************************
#   IBM Sample Source Materials
#
#   Sample source materials are supplied As-Is.
#   No warranty is expressed or implied.
#
#   Product(s): 5724-O05 soa
#
#   (C)Copyright IBM Corp. 2008
#
#   The source code for this program supplied under the terms of the
#   End User License Agreement (EULA) that accompanied this product.
#   ******************************************************************
#
#   end_generated_IBM_prolog

import time, os, sys, os.path, shutil
from java.lang import System, Runtime, String, Long, Float, Integer, Exception
from java.util import Properties
from java.io import File, FileInputStream
from pytwss.utils import _, setLogFile, log, logScreen
import initdb
import wsadmin_utils_cfg_db, wsadmin_utils_cfg_app, wsadmin_utils_cfg

COMPONENT_NAME = "sp_pc"

#
# Perform the configuration process.
# param: java.util.Properties response_properties object with the defined properties
# param: deploy_info object used to query the server or cluster members.
# return: the result "SUCCESS" "FAILURE"
#
def configure(response_properties, deploy_info, selectedFeatures_properties):
    
    # check component marker
    component_marker_path = response_properties.getProperty("COMPONENT_MARKER_PATH")
    base_path = response_properties.getProperty("TWSS_BASE_PATH")
    ddl_root = response_properties.get("DDL_PATH")
    ddl_type = response_properties.get("DDL_TYPE")
    was_home = response_properties.get("WAS_HOME")
    #marker_path = component_marker_path + os.sep + COMPONENT_NAME + ".properties"
    marker_path = selectedFeatures_properties.getProperty("Parlay_Connector")
    configMode = response_properties.get("ConfigurationMode")
    
    #if os.path.exists(marker_path) != 1:
    if marker_path == 'false':
       log("Skip Configuring: " + COMPONENT_NAME)
       return "SUCCESS"
           	
    log("Configuring: " + COMPONENT_NAME)  
    logScreen(_("SOAC7533I") %{ '1' : COMPONENT_NAME })

    # do initialization
    # Note: the ParlayConnector*.ddl and the configure* methods here are very slow, lets skip around them if we've already done them
    alreadyProcessed = initdb.getConfigProperty(response_properties, "COMMON_SP_PC_MARKER_ALREADY_PROCESSED")
    if alreadyProcessed != "TRUE":
       # parlay connector processing
       if marker_path == 'true':
           if configMode == 'Migrate':
           	# Run service specific migration ddl
           	if ddl_type == 'Db2':
        	    ddl_path = initdb.buildDDLPath(ddl_root, "PC-TWSS70DBMigDb", ddl_type)
        	    if ddl_path == "FAILURE":
            	        return "FAILURE"
        
        	    result = initdb.executeDDL(response_properties, ddl_path)        
    		    if result != "SUCCESS":
       		        return "FAILURE"
       	   else:  
               pc_ddl_path = initdb.buildDDLPath(ddl_root, "ParlayConnector", ddl_type)
               if pc_ddl_path == "FAILURE":
                   return "FAILURE"
          
               # initialize database
               result = initdb.executeDDL(response_properties, pc_ddl_path)        
               if result != "SUCCESS":
                   return "FAILURE"
             
    # set the alreadyProcessed flag
    initdb.setConfigProperty(response_properties, "COMMON_SP_PC_MARKER_ALREADY_PROCESSED", "TRUE")
    
    # Get the Scope
    scope = deploy_info.getClusterId()
    if scope == None or scope == "":
       scope = deploy_info.getCellId()
    log("Target scope: " + scope)
    
    dest_scope = deploy_info.getClusterId()
    if dest_scope == None or dest_scope == "":
       dest_scope = deploy_info.getServerIds()[0]
    log("Destination scope: " + dest_scope)

    # data source provider    
    dbType = response_properties.get("DatabaseServerType")
    provider = wsadmin_utils_cfg_db.createJDBCProvider(dbType, "${ORACLE_JDBC_DRIVER_PATH}/ojdbc14.jar", "", "false", scope)
       
    # data source(s)
    dbName = response_properties.get("DatabaseName")
    dbServer = response_properties.get("DatabaseServerHostname")
    dbPort = response_properties.get("DatabaseServerPort")
    dbUserUserName =  response_properties.get("DatabaseServerUserName")
    dbUserPassword =  response_properties.get("DatabaseServerUserPassword")
    wsadmin_utils_cfg.createJAASAuthData("PARLAYAlias",  dbUserUserName, dbUserPassword, "database and queue client identity", scope)
    dsPARLAY = wsadmin_utils_cfg_db.createGenericDatasource(dbType, "PARLAYDataSource", "jdbc/PARLAYDB", 128, provider, "PARLAYAlias", "PARLAY DataSource", dbName, scope, dbServer, dbPort)
            
    # Get the TWSSAlias   
    twss_alias = "TWSSAlias"
    
    # Get the Ear Path and Name
    pc_path = base_path + os.sep + 'parlay' + os.sep + 'P42_ParlayConnector.ear'
    pc_name = wsadmin_utils_cfg_app.getAppName(pc_path)
    
    # Install the App
    wsadmin_utils_cfg_app.installApp(pc_name, pc_path, dest_scope, db_type=dbType, alias=twss_alias)
    
    # Map the roles. TODO Correct group?  Correct EAR?
    wsadmin_utils_cfg_app.mapRole(pc_name, 'PARLAYAdministrator', all_auth="true") 
    
    return "SUCCESS"

#
# Perform the syncServers process
# param: java.util.Properties response_properties object with the defined properties 
# return: the result "SUCCESS" "FAILURE"
#
def syncServers(response_properties, deploy_info, selectedFeatures_properties):
	
    # check component marker
    component_marker_path = response_properties.getProperty("COMPONENT_MARKER_PATH")
        
    #marker_path = component_marker_path + os.sep + COMPONENT_NAME + ".properties"    
    marker_path = selectedFeatures_properties.getProperty("Parlay_Connector")
    #if os.path.exists(marker_path) != 1:
    if marker_path == 'false':
       log("Skip SyncServers: " + COMPONENT_NAME)
       return "SUCCESS"
           	
    log("SyncServers: " + COMPONENT_NAME)

    # do initialization
    logScreen(_("SOAC7533I") %{ '1' : COMPONENT_NAME })
    # more later
    
    return "SUCCESS"
