import sys.argv
import getopt
import time, os, sys, os.path, shutil
from java.lang import System, Runtime, String, Long, Float, Integer, Exception
from java.util import Properties, Set, Iterator, HashSet
from java.io import File, FileInputStream, RandomAccessFile, InputStream, InputStreamReader, BufferedReader, BufferedWriter, OutputStreamWriter, PrintWriter, FileOutputStream
from com.ibm.soa.configgui.utils import PasswordUtils, CommentedProperties
from pytwss.mbean.nodeagent_mbean import getNodeAgentInfo, restartNodeAgent
from configure import configure, syncServers
from password_utils import processPasswords
from deploy_info import getDeploymentInfo
import wsadmin_utils
import wsadmin_utils_cfg as cfg_utils
import wsadmin_utils_cfg_app as app_utils
import wsadmin_utils_cfg_db as db_utils
import wsadmin_utils_cfg_jms as jms_utils
import initdb
from pytwss.mbean import spm_jython                     # note: apparently must import this from main firststeps.py file too
from pytwss.mbean import mbean_utils                    # note: apparently must import this from main firststeps.py file too
from pytwss.mbean import dmgr_mbean                     
from pytwss.utils import _, setLogFile, log, logScreen
from javax.xml.transform.stream import StreamSource
from javax.xml.transform.stream import StreamResult
from javax.xml.parsers import DocumentBuilderFactory

def main(**kwargs):
      #
      # 1) format time in yyyymmddhhmmss
      #
      timeStr = time.strftime("%Y%m%d%H%M%S", time.localtime())
      timeStr1 = "%s" %(String(Long(System.currentTimeMillis()).toString()))
      
      #
      # 1.1) Get WAS_HOME environment variable
      #
      WAS_HOME = os.getenv('WAS_HOME')
    
      if not WAS_HOME:
         logScreen(_("SOAC7532E"))       # WAS_HOME required
         return;

      # 
      # 1.1.1 Validate the WSAdmin process is running.    
      # WSAdmin must be running to generate hints.
      log("Validating WSAdmin.")
      if (validateWsadmin() == "false"):
         log("ERROR: Verify that wsadmin can connect to a running process.")
         logScreen(_("SOAC7530E"))
         return

      wsadmin_utils.setAdminRefs((AdminConfig, AdminControl, AdminTask, AdminApp))
      cfg_utils.setAdminRefs((AdminConfig, AdminControl, AdminTask, AdminApp))
      app_utils.setAdminRefs((AdminConfig, AdminControl, AdminTask, AdminApp))
      jms_utils.setAdminRefs((AdminConfig, AdminControl, AdminTask, AdminApp))
      db_utils.setAdminRefs((AdminConfig, AdminControl, AdminTask, AdminApp))
          
      #
      # 1.2) Get additional paths derived from WAS_HOME
      # Might want to load these in another script if there are many more..
      #
      INSTALLABLE_APPS_PATH = WAS_HOME + os.sep + "installableApps" 
      TWSS_BASE_PATH = INSTALLABLE_APPS_PATH + os.sep + "TWSS-Base"
      TWSS_SERVICES_PATH = INSTALLABLE_APPS_PATH + os.sep + "TWSS-Services"
      COMPONENT_MARKER_PATH = TWSS_BASE_PATH + os.sep + "versionInfo"
      SERVICE_MARKER_PATH = TWSS_SERVICES_PATH + os.sep + "versionInfo"
      DDL_PATH = TWSS_BASE_PATH + os.sep + "database"
      DDL_SERVICES_PATH = TWSS_SERVICES_PATH + os.sep + "database"
        
      #
      # 2) create time-specific log subdirectory under firststeps
      #
      firststeps_path = WAS_HOME + File.separator + "installableApps" + File.separator + "TWSS-Base" + File.separator + "firststeps";
      logs_path = firststeps_path + File.separator + "logs" + File.separator + timeStr
      os.makedirs(logs_path);
      
      #
      # 3) set log file name to utils.py
      #
      logfile_path = logs_path + File.separator + "trace.log"
      if kwargs.has_key('nolog'):                                       # undocumented internal option for unit testing
         setLogFile(None)
      else:
         setLogFile(logfile_path);
      log("WAS_HOME=" + WAS_HOME)
      #logScreen(_("SOAC7531I") %{'1' : logfile_path})                # put logfile path onto the screen for convenience
      
      PROFILE_NAME = hintAtName(WAS_HOME, "ProfileName")                 # hintAtName calls log, so do it after the log is redirected

      #
      # 4) parse options
      #
      
      # uncomment these lines for printing arguments for debugging
      # but it may print sensitive info lie command line passwords also
      #if kwargs:
          #log(_("SOAC7522I"))
          #for key in kwargs.keys():
              #log(_("SOAC7523I") %{ '1' : key, '2' : kwargs[key] })
      
      silent_option = 'false'
      syncServers_option = 'false'
      backupdbonly_option = 'false'
      responsefile_path = firststeps_path + File.separator + "firststeps.properties";    # default response file
      
      if kwargs.has_key('silent'):
      	silent_option = 'true'
              
      if kwargs.has_key('syncServers'):
      	syncServers_option = 'true'
        
      if kwargs.has_key('backupdbonly'):
      	backupdbonly_option = 'true'
        
      if kwargs.has_key('file'):
         responsefile_path = kwargs.get('file')
         
      log("options: silent=%s syncServers=%s file=%s backupdbonly=%s" %(silent_option, syncServers_option, responsefile_path, backupdbonly_option))
      
      # 
      # 5) Enable WAS/JMX tracing
      #
      if kwargs.has_key('trace'):                                       # undocumented internal option for advanced diagnostics
         AdminControl.trace('com.ibm.*=all')
         AdminApp.trace('com.ibm.*=all')
         AdminConfig.trace('com.ibm.*=all')

      #
      # 6) copy unmodified firststeps.properties to log directory
      #
      fileName = File(responsefile_path).getName();
      if os.path.exists(responsefile_path) == 1:
         shutil.copyfile(responsefile_path, logs_path + File.separator + fileName + ".orig")
      else:                                                             # a response file MUST exist
         log("ERROR: " + responsefile_path + " " + "does not exist.")
         logScreen(_("SOAC7530E"))
         logScreen(_("SOAC7531I") %{'1' : logfile_path})                # put logfile path onto the screen for convenience
         return
      #
      # Validate the firststeps properties here!
      #
      template_path = firststeps_path + File.separator + "firststepsTemplate.properties"
      if responsefile_path == template_path or responsefile_path == "firststepsTemplate.properties":
         log("ERROR: " + template_path + " is not a valid configuration file.")
         logScreen(_("SOAC7530E"))
         logScreen(_("SOAC7531I") %{'1' : logfile_path})                # put logfile path onto the screen for convenience
         return

      validProperties = validateProperties(responsefile_path)
      if validProperties == "false":
          log("ERROR: response file has an invalid format.")
          logScreen(_("SOAC7530E"))
          logScreen(_("SOAC7531I") %{'1' : logfile_path})                # put logfile path onto the screen for convenience
          return
   
      # 7) perform the tasks
      
      if backupdbonly_option == 'true':                  # do syncServers
         #
         # Read properties file
         #
         response_properties = Properties();
         response_properties.load(FileInputStream(File(responsefile_path)));
         try:
            assignPasswords(response_properties, kwargs)
         except:
            log("A property value is malformed.  All passwords must be entered from the GUI.")
            log('ERROR: %s %s %s %s' %(sys.exc_value, str(sys.exc_info()[0]), str(sys.exc_info()[1]), str(sys.exc_info()[2].dumpStack())))
            logScreen(_("SOAC7530E"))
            return "FAILURE"
         trimProperties(response_properties)
         response_properties.put("WAS_HOME", WAS_HOME)          # so it can be checked later (don't need to save because it is recalculated each time)
         response_properties.put("LOG_PATH", logs_path)         # so it can be referenced later
         response_properties.put("COMPONENT_MARKER_PATH", COMPONENT_MARKER_PATH)
         response_properties.put("SERVICE_MARKER_PATH", SERVICE_MARKER_PATH)
         response_properties.put("TWSS_BASE_PATH", TWSS_BASE_PATH)
         response_properties.put("TWSS_SERVICES_PATH", TWSS_SERVICES_PATH)
         response_properties.put("DDL_PATH", DDL_PATH)
         response_properties.put("DDL_SERVICES_PATH", DDL_SERVICES_PATH)
         response_properties.put("PROFILE_NAME", PROFILE_NAME)
      
         db_type = response_properties.get("DatabaseServerType")
         ddl_type = "Db2";
         if db_type == "Oracle":
            ddl_type = "Ora";
         response_properties.put("DDL_TYPE", ddl_type)
         
         try:
            result = initdb.backupDatabaseOnly(response_properties)
         except:
            log('ERROR: %s %s %s %s' %(sys.exc_value, str(sys.exc_info()[0]), str(sys.exc_info()[1]), str(sys.exc_info()[2].dumpStack())))
            result = "FAILURE"
            
      elif syncServers_option == 'true':                  # do syncServers
         if os.path.exists(responsefile_path) == 1:
            shutil.copyfile(responsefile_path, logs_path + File.separator + fileName)
            
         #
         # Read properties file
         #
         response_properties = Properties();
         response_properties.load(FileInputStream(File(responsefile_path)));
         
         selectedFeatures_path = firststeps_path + File.separator + "SelectedFeatures.properties"; 
	 selectedFeatures_properties = Properties();
         selectedFeatures_properties.load(FileInputStream(File(selectedFeatures_path)));   
         
         try:
            assignPasswords(response_properties, kwargs)
         except:
            log("A property value is malformed.  All passwords must be entered from the GUI.")
            log('ERROR: %s %s %s %s' %(sys.exc_value, str(sys.exc_info()[0]), str(sys.exc_info()[1]), str(sys.exc_info()[2].dumpStack())))
            logScreen(_("SOAC7530E"))
            return "FAILURE"
         trimProperties(response_properties)
         response_properties.put("WAS_HOME", WAS_HOME)          # so it can be checked later (don't need to save because it is recalculated each time)
         response_properties.put("LOG_PATH", logs_path)         # so it can be referenced later
         response_properties.put("COMPONENT_MARKER_PATH", COMPONENT_MARKER_PATH)
         response_properties.put("SERVICE_MARKER_PATH", SERVICE_MARKER_PATH)
         response_properties.put("TWSS_BASE_PATH", TWSS_BASE_PATH)
         response_properties.put("TWSS_SERVICES_PATH", TWSS_SERVICES_PATH)
         response_properties.put("DDL_PATH", DDL_PATH)
         response_properties.put("DDL_SERVICES_PATH", DDL_SERVICES_PATH)
         response_properties.put("PROFILE_NAME", PROFILE_NAME)
         
         db_type = response_properties.get("DatabaseServerType")
         ddl_type = "Db2";
         if db_type == "Oracle":
            ddl_type = "Ora";
         response_properties.put("DDL_TYPE", ddl_type)
         
         #
         # Set administrator credentials
         #
         admin_client_username = response_properties.get("WASAdminUserName")
         admin_client_password = response_properties.get("WASAdminPassword")
         mbean_utils.setUserName(admin_client_username)
         mbean_utils.setPassword(admin_client_password)
         
         #
         # Validate deployment manager and nodeagents are running
         #
         #log("Validating dmgr, nodeagents: %s" %(processPasswords(response_properties)))
         
         try:
            ag_deploy_info = getDeploymentInfo(response_properties, "AG")
         except:
            log('ERROR: %s %s %s %s' %(sys.exc_value, str(sys.exc_info()[0]), str(sys.exc_info()[1]), str(sys.exc_info()[2].dumpStack())))
            log("ERROR: The deployment information could not be obtained.  Verify correct server/node/cluster scopes were used.")
            logScreen(_("SOAC7530E"))
            logScreen(_("SOAC7531I") %{'1' : logfile_path})                # put logfile path onto the screen for convenience
            return
         if(ag_deploy_info and validateNodeAgents(ag_deploy_info.getNodeAgentInfo()) == "false"):
            log("All the defined node agents must be started.  Please start them.")
            logScreen(_("SOAC7530E"))
            logScreen(_("SOAC7531I") %{'1' : logfile_path})                # put logfile path onto the screen for convenience
            return		
            
         try:
            sp_deploy_info = getDeploymentInfo(response_properties, "SP")
         except:
            log('ERROR: %s %s %s %s' %(sys.exc_value, str(sys.exc_info()[0]), str(sys.exc_info()[1]), str(sys.exc_info()[2].dumpStack())))
            log("ERROR: The deployment information could not be obtained.  Verify correct server/node/cluster scopes were used.")
            logScreen(_("SOAC7530E"))
            logScreen(_("SOAC7531I") %{'1' : logfile_path})                # put logfile path onto the screen for convenience
            return
         if(sp_deploy_info and validateNodeAgents(sp_deploy_info.getNodeAgentInfo()) == "false"):
            log("All the defined node agents must be started.  Please start them.")
            logScreen(_("SOAC7530E"))
            logScreen(_("SOAC7531I") %{'1' : logfile_path})                # put logfile path onto the screen for convenience
            return		
            
         try:
            spm_deploy_info = getDeploymentInfo(response_properties, "SPM")
         except:
            log('ERROR: %s %s %s %s' %(sys.exc_value, str(sys.exc_info()[0]), str(sys.exc_info()[1]), str(sys.exc_info()[2].dumpStack())))
            log("ERROR: The deployment information could not be obtained.  Verify correct server/node/cluster scopes were used.")
            logScreen(_("SOAC7530E"))
            logScreen(_("SOAC7531I") %{'1' : logfile_path})                # put logfile path onto the screen for convenience
            return
         if(spm_deploy_info and validateNodeAgents(spm_deploy_info.getNodeAgentInfo()) == "false"):
            log("All the defined node agents must be started.  Please start them.")
            logScreen(_("SOAC7530E"))
            logScreen(_("SOAC7531I") %{'1' : logfile_path})                # put logfile path onto the screen for convenience
            return		
         #
         # Need to test if this response file has already been used to do an initial configuration, otherwise an error
         #
         initial_config = response_properties.get("INITIAL_CONFIGURATION")
         if initial_config != None:
            try:
               result = syncServers(response_properties, selectedFeatures_properties)
            except:
               log('ERROR: %s %s %s %s' %(sys.exc_value, str(sys.exc_info()[0]), str(sys.exc_info()[1]), str(sys.exc_info()[2].dumpStack())))
               result = "FAILURE"
         else:
            log("The response file must be used to do the initial configuration before -syncServers can be used.")
            result = "FAILURE"
         
      else:     # do configuration
            
         #
         # 8) if interactive, start configgui (which will edit and update the response.properties file)
         #
         if silent_option == 'false':
            locale = "en_US"                    #default locale
            try:
               info_path = firststeps_path + File.separator + "FirstStepsInfo.properties";    # Installer Info Properties path
               info_properties = Properties()
               info_properties.load(FileInputStream(File(info_path)))
               locale = info_properties.get("locale")
            except:
               log("Error reading the firststepsInfo.properties file")
               
            #
            # Generate a hint for the clusterName, nodeName, serverName
            # This is a convenience to the user only.  It may not always give the desired name if there are multiple profiles, cells, nodes, servers, etc
            # Note: this assumes AppSrv01 for simplicity, and this is only a hint.  If user used some other name, they will have to fill-in themselves
            #
            clusterName = hintAtName(WAS_HOME, "Cluster")
            if clusterName != None:
               nodeName = ""
               serverName = ""
            else:
               clusterName = ""
               nodeName = hintAtName(WAS_HOME, "Node")
               serverName = hintAtName(WAS_HOME, "Server")
            ports = hintAtName(WAS_HOME, "WC_defaulthost")
            log      (_("SOAC7537I") %{'1' : clusterName, '2' : nodeName, '3' : serverName, '4' : ports})
            logScreen(_("SOAC7537I") %{'1' : clusterName, '2' : nodeName, '3' : serverName, '4' : ports})
            
            try:
               javaProg = WAS_HOME + File.separator + "java" + File.separator + "bin" + File.separator + "java"
               cmdLine = javaProg + " -jar configgui.jar @" + responsefile_path + " -lang=" + locale + " -logs=" + logs_path;
               log("Exec configgui: " + cmdLine)
               proc = Runtime.getRuntime().exec(cmdLine);
               rcode = proc.waitFor();
               log("Exec configui rcode=%s" %(rcode))
               
               #
               # Test the exit rcode, if not 0 we are not supposed to do the configuration process
               # 0 - configure
               # 1 - error (maybe DISPLAY not set)
               # 2 - save
               # 4 - cancel
               #
               if rcode != 0:
                  if rcode == 1:
                     log("Java JVM error, possibly the DISPLAY= environment variable is not setup.")
                     logScreen(_("SOAC7530E"))
                     logScreen(_("SOAC7531I") %{'1' : logfile_path})                # put logfile path onto the screen for convenience
                     return
                  elif rcode == 2:
                     log("Save clicked.  Exiting without configuring.")
                     logScreen(_("SOAC7513I"))
                     logScreen(_("SOAC7531I") %{'1' : logfile_path})                # put logfile path onto the screen for convenience
                     return;
                  elif rcode == 4:
                     log("Cancel clicked.  Exiting without configuring.")
                     logScreen(_("SOAC7513I"))
                     logScreen(_("SOAC7531I") %{'1' : logfile_path})                # put logfile path onto the screen for convenience
                     return;
                  else:
                     log("Unknown return.  Exiting without configuring.")
                     log(_("SOAC7513I"))
                     logScreen(_("SOAC7513I"))
                     logScreen(_("SOAC7531I") %{'1' : logfile_path})                # put logfile path onto the screen for convenience
                     return;

            except Exception, e:   
               log('ERROR: %s %s %s %s' %(sys.exc_value, str(sys.exc_info()[0]), str(sys.exc_info()[1]), str(sys.exc_info()[2].dumpStack())))
               log("Error occurred executing the configgui")
               logScreen(_("SOAC7530E"))
               logScreen(_("SOAC7531I") %{'1' : logfile_path})                # put logfile path onto the screen for convenience
               return
         #
         # 9) read properties file
         #
         response_properties = Properties();
         response_properties.load(FileInputStream(File(responsefile_path)))
         
         selectedFeatures_path = firststeps_path + File.separator + "SelectedFeatures.properties"; 
	 selectedFeatures_properties = Properties();
         selectedFeatures_properties.load(FileInputStream(File(selectedFeatures_path)));    
             
         try:
	    if silent_option == 'false':
                decryptPasswords(response_properties, responsefile_path)
	    else:
	        assignPasswords(response_properties, kwargs)
         except:
            log("A property value is malformed.  All passwords must be entered from the GUI.")
            log('ERROR: %s %s %s %s' %(sys.exc_value, str(sys.exc_info()[0]), str(sys.exc_info()[1]), str(sys.exc_info()[2].dumpStack())))
            logScreen(_("SOAC7530E"))
            return "FAILURE"
         trimProperties(response_properties)
         response_properties.put("WAS_HOME", WAS_HOME)          # so it can be checked later (don't need to save because it is recalculated each time)
         response_properties.put("LOG_PATH", logs_path)         # so it can be referenced later
         response_properties.put("COMPONENT_MARKER_PATH", COMPONENT_MARKER_PATH)
         response_properties.put("SERVICE_MARKER_PATH", SERVICE_MARKER_PATH)
         response_properties.put("TWSS_BASE_PATH", TWSS_BASE_PATH)
         response_properties.put("TWSS_SERVICES_PATH", TWSS_SERVICES_PATH)
         response_properties.put("DDL_PATH", DDL_PATH)
         response_properties.put("DDL_SERVICES_PATH", DDL_SERVICES_PATH)
         response_properties.put("PROFILE_NAME", PROFILE_NAME)
         
         db_type = response_properties.get("DatabaseServerType")
         ddl_type = "Db2";
         if db_type == "Oracle":
            ddl_type = "Ora";
         response_properties.put("DDL_TYPE", ddl_type)
         
         #
         # Check to see if all the scopes are None
         #
         
         ag_scope = response_properties.get("AGTargetScope")
         spm_scope = response_properties.get("SPMTargetScope")
         sp_scope = response_properties.get("SPTargetScope")

	 selectedFeaturesKeys = selectedFeatures_properties.keySet();
	 selectedFeaturesIt = selectedFeaturesKeys.iterator();
	 
	 agComponetsSet = HashSet();
	 agComponents = ["Access_Gateway_Runtime", "Audio_Call_Flow", "Address_List_Management_Flow", 
		         "Account_Management_Flow", "Call_Handling_Flow", "Call_Notification_Flow", "Multimedia_Conference_Flow",
		         "Multimedia_Messaging_Flow", "Payment_ACS_Flow", "Payment_RVCS_Flow", "Presence_Flow",
		         "Short_Messaging_Service_Flow", "Terminal_Location_Flow", "Terminal_Status_Flow", "Third_Party_Call_Flow",
		         "WAP_Push_Call_Flow"];

	 for i in range(len(agComponents)):
    	     agComponetsSet.add(agComponents[i])    			

	 SPComponetsSet = HashSet();
	 SPComponents = ["Common_Component_Applications", "Parlay_Connector", "Service_Platform_Runtime", "CN_over_SIP", "Presence_over_SIP",
		         "TS_over_SIP", "TPC_over_SIP", "Payment_PostPaid_over_Usage_Records", "ALM_over_XCAP", "SMS_over_SMPP",
		         "WAP_Push_over_SMPP", "TL_over_MLP", "MMS_over_MM7", "CH_over_Parlay", "CN_over_Parlay", "SMS_over_Parlay",
		         "TS_over_Parlay", "TL_over_Parlay", "TPC_over_Parlay"];
		

	 for j in range(len(SPComponents)):
    	     SPComponetsSet.add(SPComponents[j])   

	 SPMComponetsSet = HashSet();
	 SPMComponents = ["SPM_Runtime", "SPM_Console"];

	 for k in range(len(SPMComponents)):
    	     SPMComponetsSet.add(SPMComponents[k]) 
    
         if(ag_scope == 'None' and spm_scope == 'None' and sp_scope == 'None'):
            log("ERROR: No target scopes were selected for configuration.")
            logScreen(_("SOAC7530E"))
            logScreen(_("SOAC7531I") %{'1' : logfile_path})                # put logfile path onto the screen for convenience
            return            
         
         #check and make sure that correct target scope is provided for the selected components
         while selectedFeaturesIt.hasNext():
   	     propDisplayName = selectedFeaturesIt.next();
      	
   	     if agComponetsSet.contains(propDisplayName) == 1 and selectedFeatures_properties.get(propDisplayName) == 'true' and ag_scope == 'None':
       	         log("ERROR: Some of AG components are selected for configuration & deployment, but target scope for AG is None or incorrect. Please check the AG target scope.")
                 logScreen(_("SOAC7530E"))
                 logScreen(_("SOAC7531I") %{'1' : logfile_path})                # put logfile path onto the screen for convenience
       		 return
    
   	     if SPMComponetsSet.contains(propDisplayName) == 1 and selectedFeatures_properties.get(propDisplayName) == 'true' and spm_scope == 'None':
       		 log("ERROR: Some of SPM components are selected for configuration & deployment, but target scope for SPM is None or incorrect. Please check the AG target scope.")
                 logScreen(_("SOAC7530E"))
                 logScreen(_("SOAC7531I") %{'1' : logfile_path})                # put logfile path onto the screen for convenience
       		 return
   
   	     if SPComponetsSet.contains(propDisplayName) == 1 and selectedFeatures_properties.get(propDisplayName) == 'true' and sp_scope == 'None':
       		 log("ERROR: Some of SP components are selected for configuration & deployment, but target scope for SP is None or incorrect. Please check the SP target scope.")
                 logScreen(_("SOAC7530E"))
                 logScreen(_("SOAC7531I") %{'1' : logfile_path})                # put logfile path onto the screen for convenience
       		 return
       
         #
         # Set administrator credentials
         #
         admin_client_username = response_properties.get("WASAdminUserName")
         admin_client_password = response_properties.get("WASAdminPassword")
         mbean_utils.setUserName(admin_client_username)
         mbean_utils.setPassword(admin_client_password)
         
         #
         # Validate deployment manager and nodeagents are running
         #
         #log("Validating dmgr, nodeagents: %s" %(processPasswords(response_properties)))

         try:
            ag_deploy_info = getDeploymentInfo(response_properties, "AG")
         except:
            log('ERROR: %s %s %s %s' %(sys.exc_value, str(sys.exc_info()[0]), str(sys.exc_info()[1]), str(sys.exc_info()[2].dumpStack())))
            log("ERROR: The deployment information could not be obtained.  Verify correct server/node/cluster scopes were used.")
            logScreen(_("SOAC7530E"))
            logScreen(_("SOAC7531I") %{'1' : logfile_path})                # put logfile path onto the screen for convenience
            return
         if(ag_deploy_info and validateNodeAgents(ag_deploy_info.getNodeAgentInfo()) == "false"):
            log("All the defined node agents must be started.  Please start them.")
            logScreen(_("SOAC7530E"))
            logScreen(_("SOAC7531I") %{'1' : logfile_path})                # put logfile path onto the screen for convenience
            return
			
         try:
            sp_deploy_info = getDeploymentInfo(response_properties, "SP")
         except:
            log('ERROR: %s %s %s %s' %(sys.exc_value, str(sys.exc_info()[0]), str(sys.exc_info()[1]), str(sys.exc_info()[2].dumpStack())))
            log("ERROR: The deployment information could not be obtained.  Verify correct server/node/cluster scopes were used.")
            logScreen(_("SOAC7530E"))
            logScreen(_("SOAC7531I") %{'1' : logfile_path})                # put logfile path onto the screen for convenience
            return
         if(sp_deploy_info and validateNodeAgents(sp_deploy_info.getNodeAgentInfo()) == "false"):
            log("All the defined node agents must be started.  Please start them.")
            logScreen(_("SOAC7530E"))
            logScreen(_("SOAC7531I") %{'1' : logfile_path})                # put logfile path onto the screen for convenience
            return
			
         try:
            spm_deploy_info = getDeploymentInfo(response_properties, "SPM")
         except:
            log('ERROR: %s %s %s %s' %(sys.exc_value, str(sys.exc_info()[0]), str(sys.exc_info()[1]), str(sys.exc_info()[2].dumpStack())))
            log("ERROR: The deployment information could not be obtained.  Verify correct server/node/cluster scopes were used.")
            logScreen(_("SOAC7530E"))
            logScreen(_("SOAC7531I") %{'1' : logfile_path})                # put logfile path onto the screen for convenience
            return
         if(spm_deploy_info and validateNodeAgents(spm_deploy_info.getNodeAgentInfo()) == "false"):
            log("All the defined node agents must be started.  Please start them.")
            logScreen(_("SOAC7530E"))
            logScreen(_("SOAC7531I") %{'1' : logfile_path})                # put logfile path onto the screen for convenience
            return
			
         #
         # Set the INITIAL_CONFIGURATION=TRUE property
         # 
         initial_config = response_properties.get("INITIAL_CONFIGURATION")
         if initial_config == None:
            # append to the file, don't use Properties to save() because it re-orders the file
            try:
               log("Writing INITIAL_CONFIGURATION to response file.")
               raf = RandomAccessFile(responsefile_path, "rw")
               raf.seek(raf.length())             
               raf.writeUTF("\r\nINITIAL_CONFIGURATION=TRUE")
               raf.close()
               log("Write Successful.")
            except Exception, e:
               log('ERROR: %s %s %s %s' %(sys.exc_value, str(sys.exc_info()[0]), str(sys.exc_info()[1]), str(sys.exc_info()[2].dumpStack())))
               log("Error occurred when trying to set the INITIAL_CONFIGURATION property in the response file.")
               logScreen(_("SOAC7530E"))
               logScreen(_("SOAC7531I") %{'1' : logfile_path})                # put logfile path onto the screen for convenience
               return
         
         #
         # 10) save a copy of the response file we will actually execute
         #
         if os.path.exists(responsefile_path) == 1:
            shutil.copyfile(responsefile_path, logs_path + File.separator + fileName)

         #
         # 11) start configuration process, and display result to use on the screen and logfile
         #
         try:
             result = configure(response_properties, selectedFeatures_properties)
         except:
            log('ERROR: %s %s %s %s' %(sys.exc_value, str(sys.exc_info()[0]), str(sys.exc_info()[1]), str(sys.exc_info()[2].dumpStack())))
            result = "FAILURE"
         
         if result != "SUCCESS":
            log(_("SOAC7530E"))
            logScreen(_("SOAC7530E"))
            logScreen(_("SOAC7531I") %{'1' : logfile_path})                # put logfile path onto the screen for convenience
            return
         
         #
         # 12) syncServers
         #
         try:
            result = syncServers(response_properties, selectedFeatures_properties)
         except:
            log('ERROR: %s %s %s %s' %(sys.exc_value, str(sys.exc_info()[0]), str(sys.exc_info()[1]), str(sys.exc_info()[2].dumpStack())))
            result = "FAILURE"

         #
         # 13) restart Deployment Manager
         #
         
         cellName = AdminControl.getCell()
         nodeName = AdminControl.getNode()
         if PROFILE_NAME.startswith("Dmgr"):
            unique_list = []
            if ag_deploy_info:
               ag_nodes = ag_deploy_info.getNodeNames()
               for mem in ag_nodes:
                  if not mem in unique_list:
                     unique_list.append(mem)
            if sp_deploy_info:
               sp_nodes = sp_deploy_info.getNodeNames()   
               for mem in sp_nodes:
                  if not mem in unique_list:
                     unique_list.append(mem)
            if spm_deploy_info:
               spm_nodes = spm_deploy_info.getNodeNames()
               for mem in spm_nodes:
                  if not mem in unique_list:
                     unique_list.append(mem)
            for node in unique_list:
               restartNodeAgent(node)
            dmgr_mbean.restartDeploymentManager(nodeName, cellName)
            
         #
         # Backup the final values of the database after configure & syncServers
         #
         try:
            result = initdb.backupDatabaseOnly(response_properties)
         except:
            log('ERROR: %s %s %s %s' %(sys.exc_value, str(sys.exc_info()[0]), str(sys.exc_info()[1]), str(sys.exc_info()[2].dumpStack())))
            result = "FAILURE"
            
      timeStr2 = "%s" %(String(Long(System.currentTimeMillis()).toString()))
      durationMin = "%s" %(((Long(timeStr2).longValue()-Long(timeStr1).longValue())/1000)/60)
      durationSec = "%s" %(((Long(timeStr2).longValue()-Long(timeStr1).longValue())/1000)%60)
      
      if result == "SUCCESS":
         log("Duration %s minutes %s seconds" %(durationMin,durationSec))
         log(_("SOAC7513I"))
         logScreen(_("SOAC7513I"))
         logScreen(_("SOAC7531I") %{'1' : logfile_path})
      else:
         log("Duration %s minutes %s seconds" %(durationMin,durationSec))
         log(_("SOAC7530E"))
         logScreen(_("SOAC7530E"))
         logScreen(_("SOAC7531I") %{'1' : logfile_path})
      
      return
      
#
# validateWsadmin
# return "true" if running, "false" if not
# 
def validateWsadmin():
    log("Validating Deployment Manager")
    try:
    	AdminControl.getHost()
    except:
    	log("ERROR: Wsadmin operations cannot be performed.")
    	return "false"
    log("Wsadmin is operational")
    return "true"
     
#
# validateNodeAgents - validate that all defined nodeagents are running
# return "true" if running, "false" if not
# 
def validateNodeAgents(node_agent_info):
    log("Validating NodeAgents")
    if node_agent_info == None:
       log("There is no node agent.")
       return "true"                            # pretend it is running ok
    isRunning = node_agent_info.isRunning()
    if(isRunning == "false"):
    	log("The following node agents are stopped:  %s" % node_agent_info.getStoppedNodeAgents())
    else:
        log("All the node agents are started:  %s" % node_agent_info.getRunningNodeAgents())
    return node_agent_info.isRunning()
      
#
# validateProperties validates the response file, but not the translatable files
# return "true" or "false"
#
def validateProperties(responsefile_path):
    log("Validating Properties: " + responsefile_path)
    if os.path.exists(responsefile_path) != 1:
       log("ERROR: The response file does not exist.  " + responsefile_path);
       return "false"

    result = "true"
    response_properties = Properties();
    response_properties.load(FileInputStream(File(responsefile_path)));
    propertyOrder = response_properties.get("PropertyOrder")
    if propertyOrder == None:
       log("ERROR: the PropertyOrder key was not found in the response properties file.")
    aProps = String(propertyOrder).split(",")
    for i in range(len(aProps)):
       # each property has a property and a .type property
       name = String(aProps[i]).trim()
       valName = name
       valVal = response_properties.get(valName)
       typeName = name + ".type";
       typeVal = response_properties.get(typeName)
       if typeVal == None:
          log("ERROR: missing property " + typeName);
          result = "false"
       
       if typeVal != "Password":
           if valVal == None:
               log("ERROR: missing property " + valName)
               result = "false"
    
    return result

#
# Decrypt any password with a Password type and reenter the decrypted
# value into the Properties map.
#
def decryptPasswords(response_properties, responsefile_path):
    fsProps = CommentedProperties();
    fsProps.load(FileInputStream(File(responsefile_path)));
    passUtils = PasswordUtils()
    propertyOrder = response_properties.get("PropertyOrder")
    aProps = String(propertyOrder).split(",")
    for i in range(len(aProps)):
       # each property has a property and a .type property
       name = String(aProps[i]).trim()
       valName = name
       valVal = response_properties.get(valName)
       typeName = name + ".type";
       typeVal = response_properties.get(typeName)
       if(typeVal == "Password"):
       	 valVal2 = passUtils.decrypt(valVal)
         #log("decryptPassword %s %s" %(valVal, valVal2))
         response_properties.put(valName, valVal2)
         fsProps.setProperty(valName, "");
         fsProps.store(FileOutputStream(File(responsefile_path)));

   
def assignPasswords(response_properties, kwargs):
    propertyOrder = response_properties.get("PropertyOrder")
    aProps = String(propertyOrder).split(",")
    for i in range(len(aProps)):
       # each property has a property and a .type property
       name = String(aProps[i]).trim()
       typeName = name + ".type";
       typeVal = response_properties.get(typeName)
       if(typeVal == "Password"):
       	 valVal = kwargs[name]
         #log("decryptPassword %s %s" %(valVal, valVal2))
         response_properties.put(name, valVal)      


#
# Trims all properties.  Can't rely on GUI because of silent install.
#
def trimProperties(response_properties):
    propertyOrder = response_properties.get("PropertyOrder")
    aProps = String(propertyOrder).split(",")
    for i in range(len(aProps)):
       # each property has a property and a .type property
       name = String(aProps[i]).trim()
       val = String(response_properties.get(name)).trim()
       response_properties.put(name, val) 

#
#populate all the existing profiles
#

node_profiles = Properties();
dmgr_profiles = Properties();

def populateProfiles(rootPath):
    f = File(rootPath)
    profileRegistyPath = f.getParent() + File.separator + "properties" + File.separator + "profileRegistry.xml"
    factory = DocumentBuilderFactory.newInstance()
    builder = factory.newDocumentBuilder()

    input = FileInputStream(profileRegistyPath)
    document = builder.parse(input)
    nl = document.getElementsByTagName('profile')

    dmgr_count = 0
    node_count = 0

    for i in range(nl.length):
        e = nl.item(i);
        nnm = e.getAttributes();
        for j in range(nnm.length):
            n = nnm.item(j);
            attrname = n.getNodeName();
            if attrname == 'name':
                attrNameVal = None
                attrNameVal = n.getNodeValue();
            if attrname == 'template':
                attrTemplateVal = None
                attrTemplateVal = n.getNodeValue();
        dmgr_name = 'dmgr'
        node_name = 'node'
        if attrTemplateVal != None and attrNameVal != None:
             if attrTemplateVal.endswith("management") or attrTemplateVal.endswith("dmgr") or attrTemplateVal.endswith("dmgr.esbserver"):
                 dmgr_name += `dmgr_count`
                 log("Populating dmgr properties with key %s and value %s" %(dmgr_name, attrNameVal))
                 dmgr_profiles.put(dmgr_name, attrNameVal)
                 dmgr_count = dmgr_count + 1
             if attrTemplateVal.endswith("default") or attrTemplateVal.endswith("managed") or attrTemplateVal.endswith("default.esbserver") or attrTemplateVal.endswith("managed.esbserver"):
                 node_name += `node_count`
                 log("Populating node properties with key %s and value %s" %(node_name, attrNameVal))
                 node_profiles.put(node_name, attrNameVal)
                 node_count = node_count + 1
    node_profiles.put("NoOfNodes", (node_count))
    dmgr_profiles.put("NoOfDmgrs", (dmgr_count))
#
# findFilePath
#

def findFilePath(rootPath, fileName, filter="None"):
   f = File(rootPath)
   if f.exists() == 1:
      l = f.list()
      for i in range(len(l)):
         f2Path = rootPath + File.separator + l[i]
         if l[i] == fileName:                   # match either a file or directory
            return f2Path
         f2 = File(f2Path)
         if f2.isDirectory():
            ret = None
            if filter == "dm":         	
               if l[i] != "nodeagent" and l[i] != "wstemp" and l[i] != "temp" and l[i] != "installedApps" and l[i] != "templates" and l[i] != "tranlog" and String(l[i]).indexOf("CellManager") < 0:
                   if node_profiles.containsValue(String(l[i])) == 0:
                       ret = findFilePath(f2Path, fileName, filter="dm")    
            elif filter == "as":                     
               if l[i] != "nodeagent" and l[i] != "wstemp" and l[i] != "temp" and l[i] != "installedApps" and l[i] != "templates" and l[i] != "tranlog" and String(l[i]).indexOf("CellManager") < 0:
                   if dmgr_profiles.containsValue(String(l[i])) == 0:
                       ret = findFilePath(f2Path, fileName, filter="as")  
            else:
               if l[i] != "nodeagent" and l[i] != "wstemp" and l[i] != "temp" and l[i] != "installedApps" and l[i] != "templates" and l[i] != "tranlog" and String(l[i]).indexOf("CellManager") < 0:
                  ret = findFilePath(f2Path, fileName)
            if ret != None:
               return ret            
      return None      
   else:
      return None
                  
#
# Hint at a name for Cluster, Server, Node
# return the name 
#
# This is a convenience to the user only.  It may not always give the desired name if there are multiple profiles, cells, nodes, servers, etc
# If there are multiples defined on the system, the user will have to fill-in the appropriate one.
#
def hintAtName(WAS_HOME, type):

   #hintProfileList = [ "Dummy", "AppSrv01", "default", "Dmgr01" ]               # gives hint that prefers current profile, otherwise defaults
   hintProfileList = [ "Dummy" ]                                               # gives hint only from current profile
   
   currCell = ""
   try:
      currCell = AdminControl.getCell()
      currNode = AdminControl.getNode()
      log("hintAtName cell %s" % currCell)
      log("hintAtName node %s" % currNode)
      #logScreen("currCell %s" %(currCell))
   except:
      log("AdminControl is not available to derive a hint.")   
   
   # also pick any other profile dynamically
   profilesPath = WAS_HOME + File.separator + "profiles"
   populateProfiles(profilesPath)
   if String(currNode).indexOf("CellManager") > 0:
      binPath = findFilePath(profilesPath, currCell, filter="dm")
   else:
      binPath = findFilePath(profilesPath, currCell, filter="as")   
   log("hintAtName cellPath %s" % binPath)
   #logScreen(binPath)
   if binPath != None:
      profileName = File(binPath).getParentFile().getParentFile().getParentFile().getName()
      log("hintAtName profileName %s" % profileName)
      #logScreen(profileName)
      hintProfileList[0] = profileName;                 # replace the Dummy entry
   
   for i in range(len(hintProfileList)):
      cellsPath = WAS_HOME + File.separator + "profiles" + File.separator + hintProfileList[i] + File.separator + "config" + File.separator + "cells"
      if type == "Cluster":
         clusterPath = findFilePath(cellsPath, "cluster.xml")
         if clusterPath != None:
            clusterName = File(clusterPath).getParentFile().getName()
            return clusterName
      if type == "Node":
         nodePath = findFilePath(cellsPath, "node.xml")
         if nodePath != None:
            nodeName = File(nodePath).getParentFile().getName()
            return nodeName
      if type == "Server":
         serverPath = findFilePath(cellsPath, "server.xml")
         if serverPath != None:
            serverName = File(serverPath).getParentFile().getName()
            return serverName
      if type == "WC_defaulthost":
         serverIndexPath = findFilePath(cellsPath, "serverindex.xml")
         if serverIndexPath != None:
            portList = getWCDefaultHostPortList(serverIndexPath)
            return portList
      if type == "ProfileName":
         return profileName
         
   # end of for loop   
   return None

#
# Get the WC_defaulthost port list
#
def getWCDefaultHostPortList(serverIndexPath):
   list = ""
   lastServer = "unknown"
   try:
       inReader = BufferedReader(InputStreamReader(FileInputStream(File(serverIndexPath))))
       inLine = inReader.readLine()
       while inLine != None:
          if String(inLine).indexOf("serverName=") >= 0:
             i = String(inLine).indexOf("serverName=")+12
             j = String(inLine).indexOf("\"", i)
             lastServer = String(inLine).substring(i,j)
          if String(inLine).indexOf("\"WC_defaulthost\"") >= 0:
             inLine = String(inReader.readLine())
             i = inLine.indexOf("port=")
             if i>0:
                j = i+6;
                k = inLine.indexOf("\"",j)
                port = String(inLine.substring(j,k))
                #logScreen(port)
                if len(list) != 0:
                   list = list + ","
                list = list + ("%s=%s" %(lastServer,port))
          inLine = inReader.readLine()
       inReader.close()
   except:
       logScreen('ERROR: %s %s %s %s' %(sys.exc_value, str(sys.exc_info()[0]), str(sys.exc_info()[1]), str(sys.exc_info()[2].dumpStack())))
   return list
   

#
# Parse the options.  
#    -option    boolean parameters
#    [@]file    file parameter
#
def parseOpts(args):
     opts = { }
     for i in range(len(args)):
           opt = args[i]
           if opt.startswith("@"):
              opts["file"] = opt[1:];
           else:
	       if opt.startswith("-"):
                   opts[opt[1:]] = 'true';
           #else:
	    #  if opt != "NULL":
             #     opts["file"] = opt;
     return opts

def readPass(args):
    passwds = { }
    print "The length of args is - %i" %(len(args))
    for i in range(len(args)):
    	passwd = args[i]
    	if passwd.startswith("DatabaseServerAdministratorPassword"):
            DatabaseServerAdministratorPassword = args[i+1];
	    if DatabaseServerAdministratorPassword != "NULL":
    	        passwds["DatabaseServerAdministratorPassword"] = DatabaseServerAdministratorPassword;
	    else:
	        passwds["DatabaseServerAdministratorPassword"] = ""
	elif passwd.startswith("DatabaseServerUserPassword"):
	    DatabaseServerUserPassword = args[i+1];
	    if DatabaseServerUserPassword != "NULL":
	        passwds["DatabaseServerUserPassword"] = DatabaseServerUserPassword;
	    else:
		passwds["DatabaseServerUserPassword"] = ""
        elif passwd.startswith("WASAdminPassword"):
	    WASAdminPassword = args[i+1];
	    if WASAdminPassword != "NULL":
	        passwds["WASAdminPassword"] = WASAdminPassword;
	    else:
		passwds["WASAdminPassword"] = ""
    	elif passwd.startswith("JMXAdminPassword"):
	    JMXAdminPassword = args[i+1];
	    if JMXAdminPassword != "NULL":
	        passwds["JMXAdminPassword"] = JMXAdminPassword;
	    else:
		passwds["JMXAdminPassword"] = ""
    	elif passwd.startswith("TWSSUserPassword"):
            TWSSUserPassword = args[i+1];
	    if TWSSUserPassword != "NULL":
	        passwds["TWSSUserPassword"] = TWSSUserPassword;
	    else:
		passwds["TWSSUserPassword"] = ""
    return passwds


#
# Global: Execute if we're the main script
#
if __name__ == 'main':
        opts = parseOpts(sys.argv)
	passwds = readPass(sys.argv)
	opts.update(passwds)
        #print "%s" %(opts)
        main(**opts)
else:
    opts = parseOpts(sys.argv)
    passwds = readPass(sys.argv)
    opts.update(passwds)
    main(**opts)            

