#!/usr/bin/ksh
#   begin_generated_IBM_prolog                                          
#                                                                       
#   prolog.pro.ococ.telcosp                                             
#   ******************************************************************  
#   IBM Confidential                                                    
#                                                                       
#   OCO Source Materials                                                
#                                                                       
#   Product(s): 5724-O05 soa                                            
#                                                                       
#   Copyright IBM Corp. 2000, 2009                                      
#                                                                       
#   The source code for this program is not published or otherwise      
#   divested of its trade secrets, irrespective of what has been        
#   deposited with the US Copyright Office.                             
#   ******************************************************************  
#                                                                       
#   end_generated_IBM_prolog                                            

#------------------------------------------------------------------
# Start the firststeps.py script
# Version: 1.9
#
# To debug this script change the first line to
# "#!/usr/bin/ksh -x" or "#!/usr/bin/ksh -v".
# You may also want to comment out any "clear" commands in the
# script.
#------------------------------------------------------------------

setWASHome()
{
    CUR_DIR=`pwd`
    WAS_DIR=`dirname ${0}`/../../../
    cd ${WAS_DIR}
    WAS_HOME=`pwd`
    cd ${CUR_DIR}
}

#------------------------------------------------------------------
# Display information to user
#------------------------------------------------------------------

function displayGUI {

  print "Usage: firststeps.sh [@firststeps.properties] "
  print "		      [-silent] [-syncServers] [-DBServerAdmiPass] <DBServerAdmiPass> [-DBServerUserPass] <DBServerUserPass> [-WASAdminPass] <WASAdminPass> [-JMXAdminPass] <JMXAdminPass> [-TWSSUserPass] <TWSSUserPass>"
  print "                     [-backupdbonly]"
  print "                     [wsadmin options]"
  print " "
  print "The firststeps.properties must be a properly formed firststeps resource."
  print "The -silent : Processes the configuration silently"
  print "The -syncServers : Uses the existing configuration to update the configuration on the server instances."
  print "The -DBServerAdmiPass : Password for the database server Administrator user name."
  print "The -DBServerUserPass : Password for the databasfe server user name."
  print "The -WASAdminPass : Password for the WebSphere Administrator user ID."
  print "The -JMXAdminPass : Password for the JMX Administrator user name."
  print "The -TWSSUserPass : Password for the TWSS user name."
  print "The -backupdbonly : Will only backup the configuration tables."
  print "WSADMIN options such as -port, -user, -password can also be used."
  print ""
  print "Notes:"
  print " DB2 and Oracle Java libraries must be in the CLASSPATH environment variable."
  print " DB2 and Oracle binaries must be in the PATH environment variable."
  print " Deployment Manager and all defined nodeagents must be started."
  print ""
  print "example: ./firststeps.sh -silent -DBServerAdmiPass <password> -DBServerUserPass <password>"

} # end of function displayGUI

#------------------------------------------------------------------
# Prompt for userid's and password's if silent option is selected
#------------------------------------------------------------------

silent_option="false"

checkSilentOption() {

 for var in $*
 do
	 if [[ $var == "-silent" ]] ; then
		silent_option="true"
	 else
	 	if [[ $var == "-syncServers" ]] ; then
	 	    silent_option="true"
	 	fi
	 fi
	 if [[ $silent_option == "true" ]] ; then
     	break
     fi
 done

}

getUIDPass() {

	 typeset -i variableIndex=1
	 
	 DatabaseServerAdministratorPassword="NULL"
	 DatabaseServerUserPassword="NULL"
	 WASAdminPassword="NULL"
         JMXAdminPassword="NULL"
	 TWSSUserPassword="NULL"

	 if [[ $silent_option == "true" ]] ; then
		 for var in $*
		 do
			if [[ $var == "-DBServerAdmiPass" ]]; then 
				eval DatabaseServerAdministratorPassword=\${$(( $variableIndex + 1 ))}
				
				if [[ -z "$DatabaseServerAdministratorPassword" ]]; then
					DatabaseServerAdministratorPassword="NULL"
				fi
			elif [[ $var == "-DBServerUserPass" ]]; then
				eval DatabaseServerUserPassword=\${$(( $variableIndex + 1 ))}
				
				if [[ -z "$DatabaseServerUserPassword" ]]; then
					DatabaseServerUserPassword="NULL"
				fi
			elif [[ $var == "-WASAdminPass" ]]; then
			        eval WASAdminPassword=\${$(( $variableIndex + 1 ))}
			
				if [[ -z "$WASAdminPassword" ]]; then
					WASAdminPassword="NULL"
				fi
			elif [[ $var == "-JMXAdminPass" ]]; then
			        eval JMXAdminPassword=\${$(( $variableIndex + 1 ))}
		
				if [[ -z "$JMXAdminPassword" ]]; then
					JMXAdminPassword="NULL"
				fi
			else 
				if [[ $var == "-TWSSUserPass" ]]; then
					eval TWSSUserPassword=\${$(( $variableIndex + 1 ))}

                                        if [[ -z "$TWSSUserPassword" ]]; then
						TWSSUserPassword="NULL"
                                        fi
                                fi

			 fi
			 variableIndex=`expr $variableIndex + 1`
		 done
         fi

	if [[ $silent_option == "true" ]] ; then
		if [[ $DatabaseServerAdministratorPassword == "NULL" && $DatabaseServerUserPassword == "NULL" && $WASAdminPassword == "NULL" && $JMXAdminPassword == "NULL" && $TWSSUserPassword == "NULL" ]]; then
			print "Passwords are not supplied for the silent option. Would you like to continue? (yes or no)"
			read  answer
			case $answer in
				yes|YES|Yes|y)
					;;
				no|NO|No|n)
					exit
					;;
			esac
		fi
	fi

} # end of function getUIDPass

#------------------------------------------------------------------
# Program main code
#------------------------------------------------------------------
setWASHome

START_DIR=`pwd`

# Get the user of the process and its groups
ORIG_PATH=`echo $PATH`
OS_TYPE=`uname`
OS_PREFIX="/opt"

if [[ ${OS_TYPE} == "AIX" ]] ; then
   OS_PREFIX="/usr"
else
   if [[ ${OS_TYPE} == "SunOS" ]] ; then
       `export PATH=${ORIG_PATH}:/usr/ucb`
   fi
fi

CURR_USR=`whoami`

# Get the User and Group associated with the files under WAS_HOME
WAS_HOME_USR=`ls -ld ${WAS_HOME} | tr -s ' ' | cut -d' ' -f3`
WAS_HOME_GRP=`ls -ld ${WAS_HOME} | tr -s ' ' | cut -d' ' -f4`

if [[ ${WAS_HOME_USR} == "root" ]] ; then
   if [[ ${CURR_USR} != "root" ]]  ; then
      print "Please rerun firststeps.sh as root."
      exit
   fi
else
   if [[ $CURR_USR != "root" && $CURR_USR != $WAS_HOME_USR ]] ; then
      print "Please rerun firststeps.sh as root or $WAS_HOME_USR."
      exit
   fi
fi

export ORIGINAL_CLASSPATH=${CLASSPATH}

#------------------------------------------------------------------
# Need to add Oracle jars to classpath, if they exist.  DB2 is 
# already in the classpath from the DB2 installer.  If Oracle is 
# used by in some other directory, the user will need to manually
# update the CLASSPATH variable.  This guess is provided as a convenience.
#------------------------------------------------------------------


jarName="${OS_PREFIX}/app/oracle/product/10.2.0/db/jdbc/lib/ojdbc14.jar"

if [[ -e "$jarName" ]]
   then
      export ORIGINAL_CLASSPATH=${ORIGINAL_CLASSPATH}:${jarName}
fi
jarName="${OS_PREFIX}/app/oracle/product/10.2.0/db_1/jdbc/lib/ojdbc14.jar"
if [[ -e "$jarName" ]]
   then
      export ORIGINAL_CLASSPATH=${ORIGINAL_CLASSPATH}:${jarName}
fi
jarName="${OS_PREFIX}/app/oracle/product/10.2.0/db_2/jdbc/lib/ojdbc14.jar"
if [[ -e "$jarName" ]]
   then
      export ORIGINAL_CLASSPATH=${ORIGINAL_CLASSPATH}:${jarName}
fi
jarName="${OS_PREFIX}/app/oracle/product/10.2.0/db_3/jdbc/lib/ojdbc14.jar"
if [[ -e "$jarName" ]]
   then
      export ORIGINAL_CLASSPATH=${ORIGINAL_CLASSPATH}:${jarName}
fi
jarName="${OS_PREFIX}/oracle/product/10.2.0/db/jdbc/lib/ojdbc14.jar"
if [[ -e "$jarName" ]]
   then
      export ORIGINAL_CLASSPATH=${ORIGINAL_CLASSPATH}:${jarName}
fi
jarName="${OS_PREFIX}/oracle/product/10.2.0/db_1/jdbc/lib/ojdbc14.jar"
if [[ -e "$jarName" ]]
   then
      export ORIGINAL_CLASSPATH=${ORIGINAL_CLASSPATH}:${jarName}
fi
jarName="${OS_PREFIX}/oracle/product/10.2.0/db_2/jdbc/lib/ojdbc14.jar"
if [[ -e "$jarName" ]]
   then
      export ORIGINAL_CLASSPATH=${ORIGINAL_CLASSPATH}:${jarName}
fi
jarName="${OS_PREFIX}/oracle/product/10.2.0/db_3/jdbc/lib/ojdbc14.jar"
if [[ -e "$jarName" ]]
   then
      export ORIGINAL_CLASSPATH=${ORIGINAL_CLASSPATH}:${jarName}
fi
jarName="${OS_PREFIX}/app/oracle/product/11.1.0/db/jdbc/lib/ojdbc5.jar"
if [[ -e "$jarName" ]]
   then
      export ORIGINAL_CLASSPATH=${ORIGINAL_CLASSPATH}:${jarName}
fi
jarName="${OS_PREFIX}/app/oracle/product/11.1.0/db_1/jdbc/lib/ojdbc5.jar"
if [[ -e "$jarName" ]]
   then
      export ORIGINAL_CLASSPATH=${ORIGINAL_CLASSPATH}:${jarName}
fi
jarName="${OS_PREFIX}/app/oracle/product/11.1.0/db_2/jdbc/lib/ojdbc5.jar"
if [[ -e "$jarName" ]]
   then
      export ORIGINAL_CLASSPATH=${ORIGINAL_CLASSPATH}:${jarName}
fi
jarName="${OS_PREFIX}/app/oracle/product/11.1.0/db_3/jdbc/lib/ojdbc5.jar"
if [[ -e "$jarName" ]]
   then
      export ORIGINAL_CLASSPATH=${ORIGINAL_CLASSPATH}:${jarName}
fi
jarName="${OS_PREFIX}/oracle/product/11.1.0/db/jdbc/lib/ojdbc5.jar"
if [[ -e "$jarName" ]]
   then
      export ORIGINAL_CLASSPATH=${ORIGINAL_CLASSPATH}:${jarName}
fi
jarName="${OS_PREFIX}/oracle/product/11.1.0/db_1/jdbc/lib/ojdbc5.jar"
if [[ -e "$jarName" ]]
   then
      export ORIGINAL_CLASSPATH=${ORIGINAL_CLASSPATH}:${jarName}
fi
jarName="${OS_PREFIX}/oracle/product/11.1.0/db_2/jdbc/lib/ojdbc5.jar"
if [[ -e "$jarName" ]]
   then
      export ORIGINAL_CLASSPATH=${ORIGINAL_CLASSPATH}:${jarName}
fi
jarName="${OS_PREFIX}/oracle/product/11.1.0/db_3/jdbc/lib/ojdbc5.jar"
if [[ -e "$jarName" ]]
   then
      export ORIGINAL_CLASSPATH=${ORIGINAL_CLASSPATH}:${jarName}
fi
jarName="${OS_PREFIX}/app/oracle/product/10.2.0/client/jdbc/lib/ojdbc14.jar"
if [[ -e "$jarName" ]]
   then
      export ORIGINAL_CLASSPATH=${ORIGINAL_CLASSPATH}:${jarName}
fi
jarName="${OS_PREFIX}/app/oracle/product/10.2.0/client_1/jdbc/lib/ojdbc14.jar"
if [[ -e "$jarName" ]]
   then
      export ORIGINAL_CLASSPATH=${ORIGINAL_CLASSPATH}:${jarName}
fi
jarName="${OS_PREFIX}/app/oracle/product/10.2.0/client_2/jdbc/lib/ojdbc14.jar"
if [[ -e "$jarName" ]]
   then
      export ORIGINAL_CLASSPATH=${ORIGINAL_CLASSPATH}:${jarName}
fi
jarName="${OS_PREFIX}/app/oracle/product/10.2.0/client_3/jdbc/lib/ojdbc14.jar"
if [[ -e "$jarName" ]]
   then
      export ORIGINAL_CLASSPATH=${ORIGINAL_CLASSPATH}:${jarName}
fi
jarName="${OS_PREFIX}/oracle/product/10.2.0/client/jdbc/lib/ojdbc14.jar"
if [[ -e "$jarName" ]]
   then
      export ORIGINAL_CLASSPATH=${ORIGINAL_CLASSPATH}:${jarName}
fi
jarName="${OS_PREFIX}/oracle/product/10.2.0/client_1/jdbc/lib/ojdbc14.jar"
if [[ -e "$jarName" ]]
   then
      export ORIGINAL_CLASSPATH=${ORIGINAL_CLASSPATH}:${jarName}
fi
jarName="${OS_PREFIX}/oracle/product/10.2.0/client_2/jdbc/lib/ojdbc14.jar"
if [[ -e "$jarName" ]]
   then
      export ORIGINAL_CLASSPATH=${ORIGINAL_CLASSPATH}:${jarName}
fi
jarName="${OS_PREFIX}/oracle/product/10.2.0/client_3/jdbc/lib/ojdbc14.jar"
if [[ -e "$jarName" ]]
   then
      export ORIGINAL_CLASSPATH=${ORIGINAL_CLASSPATH}:${jarName}
fi
jarName="${OS_PREFIX}/app/oracle/product/11.1.0/client/jdbc/lib/ojdbc5.jar"
if [[ -e "$jarName" ]]
   then
      export ORIGINAL_CLASSPATH=${ORIGINAL_CLASSPATH}:${jarName}
fi
jarName="${OS_PREFIX}/app/oracle/product/11.1.0/client_1/jdbc/lib/ojdbc5.jar"
if [[ -e "$jarName" ]]
   then
      export ORIGINAL_CLASSPATH=${ORIGINAL_CLASSPATH}:${jarName}
fi
jarName="${OS_PREFIX}/app/oracle/product/11.1.0/client_2/jdbc/lib/ojdbc5.jar"
if [[ -e "$jarName" ]]
   then
      export ORIGINAL_CLASSPATH=${ORIGINAL_CLASSPATH}:${jarName}
fi
jarName="${OS_PREFIX}/app/oracle/product/11.1.0/client_3/jdbc/lib/ojdbc5.jar"
if [[ -e "$jarName" ]]
   then
      export ORIGINAL_CLASSPATH=${ORIGINAL_CLASSPATH}:${jarName}
fi
jarName="${OS_PREFIX}/oracle/product/11.1.0/client/jdbc/lib/ojdbc5.jar"
if [[ -e "$jarName" ]]
   then
      export ORIGINAL_CLASSPATH=${ORIGINAL_CLASSPATH}:${jarName}
fi
jarName="${OS_PREFIX}/oracle/product/11.1.0/client_1/jdbc/lib/ojdbc5.jar"
if [[ -e "$jarName" ]]
   then
      export ORIGINAL_CLASSPATH=${ORIGINAL_CLASSPATH}:${jarName}
fi
jarName="${OS_PREFIX}/oracle/product/11.1.0/client_2/jdbc/lib/ojdbc5.jar"
if [[ -e "$jarName" ]]
   then
      export ORIGINAL_CLASSPATH=${ORIGINAL_CLASSPATH}:${jarName}
fi
jarName="${OS_PREFIX}/oracle/product/11.1.0/client_3/jdbc/lib/ojdbc5.jar"
if [[ -e "$jarName" ]]
   then
      export ORIGINAL_CLASSPATH=${ORIGINAL_CLASSPATH}:${jarName}
fi

db2Dir="${OS_PREFIX}/ibm/db2/V9.5"
if [[ -e "$db2Dir" ]]
   then
      export ORIGINAL_CLASSPATH=${ORIGINAL_CLASSPATH}:${db2Dir}/java/db2java.zip:${db2Dir}/java/db2jcc.jar:${db2Dir}/java/sqlj.zip:${db2Dir}/java/Common.jar:${db2Dir}/java/db2jcc_license_cu.jar
fi
db2Dir="${OS_PREFIX}/ibm/db2/V9.1"
if [[ -e "$db2Dir" ]]
   then
      export ORIGINAL_CLASSPATH=${ORIGINAL_CLASSPATH}:${db2Dir}/java/db2java.zip:${db2Dir}/java/db2jcc.jar:${db2Dir}/java/sqlj.zip:${db2Dir}/java/Common.jar:${db2Dir}/java/db2jcc_license_cu.jar
fi
db2Dir="${OS_PREFIX}/IBM/db2/V8.2"
if [[ -e "$db2Dir" ]]
   then
      export ORIGINAL_CLASSPATH=${ORIGINAL_CLASSPATH}:${db2Dir}/java/db2java.zip:${db2Dir}/java/db2jcc.jar:${db2Dir}/java/sqlj.zip:${db2Dir}/java/Common.jar:${db2Dir}/java/db2jcc_license_cu.jar
fi
db2Dir="${OS_PREFIX}/IBM/db2/V8.1"
if [[ -e "$db2Dir" ]]
   then
      export ORIGINAL_CLASSPATH=${ORIGINAL_CLASSPATH}:${db2Dir}/java/db2java.zip:${db2Dir}/java/db2jcc.jar:${db2Dir}/java/sqlj.zip:${db2Dir}/java/Common.jar:${db2Dir}/java/db2jcc_license_cu.jar
fi
db2Dir="${OS_PREFIX}/ibm/db2/sqllib"
if [[ -e "$db2Dir" ]]
   then
      export ORIGINAL_CLASSPATH=${ORIGINAL_CLASSPATH}:${db2Dir}/java/db2java.zip:${db2Dir}/java/db2jcc.jar:${db2Dir}/java/sqlj.zip:${db2Dir}/java/Common.jar:${db2Dir}/java/db2jcc_license_cu.jar
fi
db2Dir="${OS_PREFIX}/app/ibm/db2/V9.5"
if [[ -e "$db2Dir" ]]
   then
      export ORIGINAL_CLASSPATH=${ORIGINAL_CLASSPATH}:${db2Dir}/java/db2java.zip:${db2Dir}/java/db2jcc.jar:${db2Dir}/java/sqlj.zip:${db2Dir}/java/Common.jar:${db2Dir}/java/db2jcc_license_cu.jar
fi
db2Dir="${OS_PREFIX}/app/ibm/db2/V9.1"
if [[ -e "$db2Dir" ]]
   then
      export ORIGINAL_CLASSPATH=${ORIGINAL_CLASSPATH}:${db2Dir}/java/db2java.zip:${db2Dir}/java/db2jcc.jar:${db2Dir}/java/sqlj.zip:${db2Dir}/java/Common.jar:${db2Dir}/java/db2jcc_license_cu.jar
fi
db2Dir="${OS_PREFIX}/app/IBM/db2/V8.2"
if [[ -e "$db2Dir" ]]
   then
      export ORIGINAL_CLASSPATH=${ORIGINAL_CLASSPATH}:${db2Dir}/java/db2java.zip:${db2Dir}/java/db2jcc.jar:${db2Dir}/java/sqlj.zip:${db2Dir}/java/Common.jar:${db2Dir}/java/db2jcc_license_cu.jar
fi
db2Dir="${OS_PREFIX}/app/IBM/db2/V8.1"
if [[ -e "$db2Dir" ]]
   then
      export ORIGINAL_CLASSPATH=${ORIGINAL_CLASSPATH}:${db2Dir}/java/db2java.zip:${db2Dir}/java/db2jcc.jar:${db2Dir}/java/sqlj.zip:${db2Dir}/java/Common.jar:${db2Dir}/java/db2jcc_license_cu.jar
fi
db2Dir="${OS_PREFIX}/app/ibm/db2/sqllib"
if [[ -e "$db2Dir" ]]
   then
      export ORIGINAL_CLASSPATH=${ORIGINAL_CLASSPATH}:${db2Dir}/java/db2java.zip:${db2Dir}/java/db2jcc.jar:${db2Dir}/java/sqlj.zip:${db2Dir}/java/Common.jar:${db2Dir}/java/db2jcc_license_cu.jar
fi
db2Dir="/home/db2inst1/sqllib"
if [[ -e "$db2Dir" ]]
   then
      export ORIGINAL_CLASSPATH=${ORIGINAL_CLASSPATH}:${db2Dir}/java/db2java.zip:${db2Dir}/java/db2jcc.jar:${db2Dir}/java/sqlj.zip:${db2Dir}/java/Common.jar:${db2Dir}/java/db2jcc_license_cu.jar
fi

cd ${WAS_HOME}/installableApps/TWSS-Base/firststeps

if [[ $1 == "-help" ]]
   then
     	 displayGUI
   else
        checkSilentOption $*
     	getUIDPass $*
	if [[ $silent_option == "true" ]] ; then
		${WAS_HOME}/bin/wsadmin.sh -wsadmin_classpath ${WAS_HOME}/installableApps/TWSS-Base/firststeps/spm-runtime.jar:${WAS_HOME}/installableApps/TWSS-Base/firststeps/spm-ws-admin-client.jar:${WAS_HOME}/installableApps/TWSS-Base/firststeps/spm-ws-access-client.jar:${WAS_HOME}/installableApps/TWSS-Base/firststeps/configgui.jar:${ORIGINAL_CLASSPATH} -f ${WAS_HOME}/installableApps/TWSS-Base/firststeps/firststeps.py $1 $2 $3 $4 $5 $6 $7 $8 $9 DatabaseServerAdministratorPassword $DatabaseServerAdministratorPassword DatabaseServerUserPassword $DatabaseServerUserPassword WASAdminPassword $WASAdminPassword JMXAdminPassword $JMXAdminPassword TWSSUserPassword $TWSSUserPassword
	else
		${WAS_HOME}/bin/wsadmin.sh -wsadmin_classpath ${WAS_HOME}/installableApps/TWSS-Base/firststeps/spm-runtime.jar:${WAS_HOME}/installableApps/TWSS-Base/firststeps/spm-ws-admin-client.jar:${WAS_HOME}/installableApps/TWSS-Base/firststeps/spm-ws-access-client.jar:${WAS_HOME}/installableApps/TWSS-Base/firststeps/configgui.jar:${ORIGINAL_CLASSPATH} -f ${WAS_HOME}/installableApps/TWSS-Base/firststeps/firststeps.py $1 $2 $3 $4 $5 $6 $7 $8 $9
	fi
fi

cd ${START_DIR}

exit
