#   begin_generated_IBM_prolog
#
#   prolog.pro.samp.telcosp
#   ******************************************************************
#   IBM Sample Source Materials
#
#   Sample source materials are supplied As-Is.
#   No warranty is expressed or implied.
#
#   Product(s): 5724-O05 soa
#
#   (C)Copyright IBM Corp. 2008
#
#   The source code for this program supplied under the terms of the
#   End User License Agreement (EULA) that accompanied this product.
#   ******************************************************************
#
#   end_generated_IBM_prolog

import time, os, sys, os.path, shutil
from java.lang import System, Runtime, String, Long, Float, Integer, Exception
from java.util import Properties
from java.io import File, FileInputStream
from pytwss.utils import _, setLogFile, log, logScreen
import wsadmin_utils_cfg_app

COMPONENT_NAME = "PX21_CH_FLOW"

#
# Perform the configuration process.
# param: java.util.Properties response_properties object with the defined properties
# param: deploy_info object used to query the server or cluster members.
# return: the result "SUCCESS" "FAILURE"
#
def configure(response_properties, deploy_info, selectedFeatures_properties):
	
    # Initialize variables.
    component_marker_path = response_properties.getProperty("COMPONENT_MARKER_PATH")      
    #marker_path = component_marker_path + os.sep + COMPONENT_NAME + ".properties"
    marker_path = selectedFeatures_properties.getProperty("Call_Handling_Flow")
    base_path = response_properties.get("TWSS_BASE_PATH")
    
    # Check Marker 
    #if os.path.exists(marker_path) != 1:
    if marker_path == 'false':
       log("Skip Configuring: " + COMPONENT_NAME)
       return "SUCCESS"
           	
    log("Configuring: " + COMPONENT_NAME)
    logScreen(_("SOAC7533I") %{ '1' : COMPONENT_NAME })
    
    # Get the Cluster or Cell id.
    scope = deploy_info.getClusterId()
    if scope == None or scope == "":
       scope = deploy_info.getCellId()
    log("Target scope: " + scope)

    # Get the Cluster or Server Id typically used for application deployment.
    dest_scope = deploy_info.getClusterId()
    if dest_scope == None or dest_scope == "":
       dest_scope = deploy_info.getServerIds()[0]
    log("Destination scope: " + dest_scope)

    # The name of the SCA module.
    module_name = COMPONENT_NAME

    # The applicationName of the SCA module.
    app_name = module_name + 'App'

    # The path of the Flow EAR.
    ear_path = base_path + os.sep + COMPONENT_NAME + ".ear"
    
    # Deploy the Flow EAR.
    wsadmin_utils_cfg_app.installApp (app_name, ear_path, dest_scope)
    
    # Map roles to users and groups.
    wsadmin_utils_cfg_app.mapRole(app_name, 'CallHandlingServiceRole', all_auth="true")
        
    #Modify the URL for the Service Policy Manager.
    pol_ret_prop = 'PolicyRetrieval.servicePolicyManagerEndpoint'
    pol_ret_val = response_properties.get('SPMPolicyAccessURL')
    wsadmin_utils_cfg_app.modifySCAProperty(module_name, app_name, pol_ret_prop, pol_ret_val)
    
    #Modify the URL for the address list manager. Refer to the topic Finding the IBM XDMS XCAP root for help with locating this value.
    group_res_prop = 'GroupResolution.groupResolutionEndpoint'
    group_res_val = response_properties.get('GroupResolutionURL')
    wsadmin_utils_cfg_app.modifySCAProperty(module_name, app_name, group_res_prop, group_res_val)    

    #Modify the administrator user name that is used to submit a JMX alarm when security is enabled.
    jmx_user_prop = 'JMXNotification.adminUserName'
    jmx_user_val = response_properties.get('JMXAdminUserName')
    wsadmin_utils_cfg_app.modifySCAProperty(module_name, app_name, jmx_user_prop, jmx_user_val)
    
    #Modify the password corresponding to the administrator user name that is used to submit a JMX alarm when security is enabled. The password must be encrypted using the AdminTool.jar.
    jmx_pass_prop = 'JMXNotification.adminPassword'
    jmx_pass_val = response_properties.get('JMXAdminPassword')
    wsadmin_utils_cfg_app.modifySCAProperty(module_name, app_name, jmx_pass_prop, jmx_pass_val)
    
    # Modify the MessageRemoval.requester.requesterExceptionList property.
    msg_rm_prop = 'MsgElementRemover.request.requesterExceptionList'
    msg_rm_val = response_properties.get('TWSSUserName')
    wsadmin_utils_cfg_app.modifySCAProperty(module_name, app_name, msg_rm_prop, msg_rm_val)

    return "SUCCESS"

#
# Perform the syncServers process
# param: java.util.Properties response_properties object with the defined properties 
# return: the result "SUCCESS" "FAILURE"
#
def syncServers(response_properties, deploy_info, selectedFeatures_properties):
	
    # check component marker
    component_marker_path = response_properties.getProperty("COMPONENT_MARKER_PATH")
        
    #marker_path = component_marker_path + os.sep + COMPONENT_NAME + ".properties"   
    marker_path = selectedFeatures_properties.getProperty("Call_Handling_Flow") 
    #if os.path.exists(marker_path) != 1:
    if marker_path == 'false':
       log("Skip SyncServers: " + COMPONENT_NAME)
       return "SUCCESS"
           	
    log("SyncServers: " + COMPONENT_NAME)

    # do initialization
    logScreen(_("SOAC7533I") %{ '1' : COMPONENT_NAME })
    # more later
    
    return "SUCCESS"