import sys.argv
import getopt
import time, os, sys, os.path, shutil, jarray, os.sep
from java.lang import System, Runtime, String, Long, Float, Integer, Exception
from java.util import Properties
from java.io import File, FileInputStream, InputStream, InputStreamReader, BufferedReader, BufferedWriter, OutputStreamWriter, PrintWriter, FileOutputStream

from com.ibm.twss.spm.admin.common import ScopedPolicy
from com.ibm.twss.spm.admin.common import PolicyValue
from com.ibm.twss.spm.mbean import SPMJython

from password_utils import processPasswords

from pytwss.mbean import spm_jython
from pytwss.utils import _, setLogFile, log, logScreen, execUnixCmd, hasGroupPrivileges

#
# Initialize the database.
# Utilize the response_properties to check for the database client software, and then initialize the database instance.
# param: the configuration parameters
# return: the result "SUCCESS" "FAILURE"
#
def initdb(response_properties):
    log("initdb")
    logScreen(_("SOAC7533I") %{ '1' : "initdb" })
    
    #
    # Extract the necessary information from the properties
    #
    dbServer = response_properties.get("DatabaseServerHostname");
    dbPort = response_properties.get("DatabaseServerPort");
    dbType = response_properties.get("DatabaseServerType");
    dbMode = response_properties.get("DatabaseServerAccessMode")
    dbPath = response_properties.get("DatabaseDriverRootPath")
    dbAdminUserName = response_properties.get("DatabaseServerAdministratorUsername")
    dbAdminPassword = response_properties.get("DatabaseServerAdministratorPassword")
    dbUserUserName =  response_properties.get("DatabaseServerUserName")
    dbUserPassword =  response_properties.get("DatabaseServerUserPassword")
    dbName = response_properties.get("DatabaseName")
    dbLocale = response_properties.get("DatabaseLocale")
    configMode = response_properties.get("ConfigurationMode")
    osType = System.getProperty("os.name")
    dbAlias = dbName + "A"                     # the alias name is calculated from the database name, but may not be consistent with crtsrv* name.  no longer used.
    WAS_HOME = response_properties.get("WAS_HOME")
    log("dbServer %s dbPort %s dbType %s dbMode %s dbPath %s dbAdmin %s dbUser %s dbName %s dbLocale %s osType %s" 
       %(dbServer, dbPort, dbType, dbMode, dbPath, dbAdminUserName, dbUserUserName, dbName, dbLocale, osType))
    log("ConfigurationMode: %s" %(configMode))
  
    #
    # Check ORIGINAL_CLASSPATH here for database jars.  This classpath is passed wsadmin and end up in the runtime classpath
    #
    ocp = os.getenv("ORIGINAL_CLASSPATH")
    log("ORIGINAL_CLASSPATH %s" %(ocp))
    if dbType == "DB2":
       if String(ocp).indexOf("db2java") < 0:
          log("ERROR: DB2 driver (db2java.zip) jars must be in the classpath.")
          return "FAILURE"
          
    if dbType == "Oracle":
       if String(ocp).indexOf("ojdbc") < 0:
          log("ERROR: Oracle driver (ojdbcxx.jar) jars must be in the classpath.")
          return "FAILURE"

    #
    # Enable SPMJython spm_jython, so we can backup any existing tables, and login as User (not Admin)
    # For Oracle:  set CLASSPATH=c:\oracle\product\10.2.0\db_1\jdbc\lib\ojdbc14.jar;
    # FOR DB2:     set CLASSPATH=CLASSPATH=C:\db2\SQLLIB\java\db2java.zip;C:\db2\SQLLIB\java\db2jcc.jar;C:\db2\SQLLIB\java\sqlj.zip;C:\db2\SQLLIB\bin;C:\db2\SQLLIB\java\common.jar;C:\db2\SQLLIB\java\db2jcc_license_cisuz.jar;C:\db2\SQLLIB\java\db2jcc_license_cu.jar
    #    DB2 classpath automatically setup by DB2 install
    #
    try:
       log("SPMJython.initialize %s %s %s %s %s %s" %(dbType, dbServer, dbPort, dbAlias, dbName, dbAdminUserName))
       spm_jython.initialize(dbType, dbServer, dbPort, dbAlias, dbName, dbAdminUserName, dbAdminPassword)
       log("SPMJython.initialize complete")
    except Exception,e:
       log("ERROR: Cannot connect to the database as administrator. Please verify the database name or create as instructed by the InfoCenter. (run crtsrv*.sh) (%s)" %(e.getMessage()));
       return "FAILURE"
       
    try:
       log("SPMJython.initialize %s %s %s %s %s %s" %(dbType, dbServer, dbPort, dbAlias, dbName, dbUserUserName))
       spm_jython.initialize(dbType, dbServer, dbPort, dbAlias, dbName, dbUserUserName, dbUserPassword)
       log("SPMJython.initialize complete")
       if configMode != 'Migrate':
           backupdb(response_properties, "backup.ddl")
    except Exception,e:
       log("ERROR: Cannot connect to the database as user, therefore can not backup tables or execute configuration.  Please verify the database name or create as instructed by the InfoCenter. (run crtsrv*.sh) (%s)" %(e.getMessage()));
       return "FAILURE"
       
    try:
    	if configMode != 'Migrate':
            log("SPMJython.isValidSchema %s" %("ESBLOG"))
            result = spm_jython.isValidSchema("ESBLOG")
            log("SPMJython result=%s" %(result))
            if result==0:
                log("ERROR: The schema is not valid.  Please verify the database name or create as instructed by the InfoCenter. (run crtsrv*.sh) (result=%s)" %(result));
                return "FAILURE"
    except Exception,e:
       log("ERROR: The schema is not valid.  Please verify the database name or create as instructed by the InfoCenter. (run crtsrv*.sh) (%s)" %(e.getMessage()));
       return "FAILURE"
    
    #
    # If ConfigurationMode == 'Initial', we want to reset all DDL tables (even policy tables and CONFIGPROPERTIES)
    # If ConfigurationMode == 'Update', we don't want to reset configuration tables or state tables for ESB, SP, Services
    # If ConfigurationMode == 'Migrate', we want to reset state tables, but not configuration or policy tables.
    # commenting Migration option, AS a part of this fix, we will incorparte the migration ddl's instead of dropping the tables
    if configMode == 'Initial':
       # Not an Update or Migration.  Force Initial values for all tables.
       log("Dropping Configuration and Policy Tables...")
       twss_base_path = response_properties.get("TWSS_BASE_PATH")
       ddl_type = response_properties.get("DDL_TYPE")
       dropconfig_path = buildDDLPath(twss_base_path + File.separator + "firststeps", "dropconfig", ddl_type)
       if (dropconfig_path == "FAILURE"):
          return "FAILURE"
       executeDDL (response_properties, dropconfig_path)
       
       log("Dropping State Tables...")
       twss_base_path = response_properties.get("TWSS_BASE_PATH")
       ddl_type = response_properties.get("DDL_TYPE")
       dropstate_path = buildDDLPath(twss_base_path + File.separator + "firststeps", "dropstate", ddl_type)
       if (dropconfig_path == "FAILURE"):
          return "FAILURE"
       executeDDL (response_properties, dropstate_path)
          
    #elif configMode == 'Migrate':
       #log("Dropping State Tables...")
       #twss_base_path = response_properties.get("TWSS_BASE_PATH")
       #ddl_type = response_properties.get("DDL_TYPE")
       #dropstate_path = buildDDLPath(twss_base_path + File.separator + "firststeps", "dropstate", ddl_type)
       #if (dropconfig_path == "FAILURE"):
          #return "FAILURE"
       #executeDDL (response_properties, dropstate_path)
    else:
       log("Preserving Configuration and Policy Tables...")
       log("Preserving State Tables...")
   
    #
    # we're done
    #
    return "SUCCESS"
    
#
# Cleanup this script at the end.
# param response_properties is the configuration parameters
#
def termdb(response_properties):
   log("")              # blank line in log
   log("termdb")
   log("")
   return

#
# Execute a DDL file on the database, and return the error  (Note: DDLs if they exist, are always successful, but errors are logged).
# return: "SUCCESS" or "FAILURE"
#
def executeDDL(response_properties, ddlPath):
    log("executeDDL file: %s" %(ddlPath))
    
    if os.path.exists(ddlPath) != 1:
       log("ERROR: DDL file does not exist.")
       return "FAILURE"
    
    result = spm_jython.executeDDL(ddlPath);
    log("result:%s" %(result));
    
    return "SUCCESS"

#
# set a property in the CONFIGPROPERTIES table.  
# param: response_properties configuration properties
# param: key the key
# param: value the value
# return: void
#
def setConfigProperty(response_properties, key, value):
    # special handling just like CommonSettingsMBean.setProperty
    if value == None:
       value = "<null>"
      
    if value == "":
       value = "<emptystring>" 

    keystr = String(key)
    if keystr.indexOf("assword") == -1 and keystr.indexOf("ALREADY_PROCESSED") == -1:
       log("setConfigProperty: %s=%s" %(key, value))
    else:
       log("setConfigProperty: %s=%s" %(key, "*"))
    
    # Note: this function intentially does both INSERT and UPDATE since the semantics are to set the property to the new value. If the record already exists
    # INSERT will fail, and so UPDATE will put the new value there.  If the record doesn't exist, INSERT will create it, and UPDATE will redundantly set to the same/new value
    # we could make the argument that we shouldn't do the UPDATE unless the ConfigurationMode 'Initial' is set (in order to reset back to the initial value)
    # however, The properties that are set with this method usually have data from the input screen, which we always want to update with the latest input value.

    spm_jython.setConfigProperty(key, value)
    
    return None

#
# Get a property from CONFIGPROPERTIES table
# param: response_properties configuration properties
# param: key the key
# param: the result string
#
def getConfigProperty(response_properties, key):
    log("getConfigProperty: %s" %(key))

    result = spm_jython.getConfigProperty(key)
    
    # special handling just like CommonSettingsMBean.getProperty
    if result != None and result == "<null>":
       result = None
       
    if result != None and result == "<emptystring>":
       result = ""
    
    return result
    
#
# Remove a property from CONFIGPROPERTIES table
# param: response_properties configuration properties
# param: key the key
# param: the result string
#
def removeConfigProperty(response_properties, key):
    log("removeConfigProperty: %s" %(key))
    
    spm_jython.removeConfigProperty(key)
    
    return None
    
#
#  Backup the database only.
#  param: response properties
#
def backupDatabaseOnly(response_properties):
    #log("Backup Database Only: %s" % processPasswords(response_properties))
    logScreen(_("SOAC7533I") %{ '1' : "initdb.backupdb" })
    
    #
    # Extract the necessary information from the properties
    #
    dbServer = response_properties.get("DatabaseServerHostname");
    dbPort = response_properties.get("DatabaseServerPort");
    dbType = response_properties.get("DatabaseServerType");
    dbMode = response_properties.get("DatabaseServerAccessMode")
    dbPath = response_properties.get("DatabaseDriverRootPath")
    dbAdminUserName = response_properties.get("DatabaseServerAdministratorUsername")
    dbAdminPassword = response_properties.get("DatabaseServerAdministratorPassword")
    dbUserUserName =  response_properties.get("DatabaseServerUserName")
    dbUserPassword =  response_properties.get("DatabaseServerUserPassword")
    dbName = response_properties.get("DatabaseName")
    dbLocale = response_properties.get("DatabaseLocale")
    configMode = response_properties.get("ConfigurationMode")
    osType = System.getProperty("os.name")
    dbAlias = dbName + "A"                     # the alias name is calculated from the database name, but may not be consistent with crtsrv* name.  no longer used.
    WAS_HOME = response_properties.get("WAS_HOME")
    log("dbServer %s dbPort %s dbType %s dbMode %s dbPath %s dbAdmin %s dbUser %s dbName %s dbLocale %s osType %s" 
       %(dbServer, dbPort, dbType, dbMode, dbPath, dbAdminUserName, dbUserUserName, dbName, dbLocale, osType))
    log("ConfigurationMode: %s" %(configMode))
    
    #
    # Note: dbMode and dbLocale are not actually used here but they are in the CDD and may be needed at some point for database configuration, so I left them here.
    #
  
    #
    # Check ORIGINAL_CLASSPATH here for database jars.  This classpath is passed wsadmin and end up in the runtime classpath
    #
    ocp = os.getenv("ORIGINAL_CLASSPATH")
    log("ORIGINAL_CLASSPATH %s" %(ocp))
    if dbType == "DB2":
       if String(ocp).indexOf("db2java") < 0:
          log("ERROR: DB2 driver (db2java.zip) jars must be in the classpath.")
          return "FAILURE"
          
    if dbType == "Oracle":
       if String(ocp).indexOf("ojdbc") < 0:
          log("ERROR: Oracle driver (ojdbcxx.jar) jars must be in the classpath.")
          return "FAILURE"

    #
    # Enable SPMJython spm_jython, so we can backup any existing tables, and login as User (not Admin)
    # For Oracle:  set CLASSPATH=c:\oracle\product\10.2.0\db_1\jdbc\lib\ojdbc14.jar;
    # FOR DB2:     set CLASSPATH=CLASSPATH=C:\db2\SQLLIB\java\db2java.zip;C:\db2\SQLLIB\java\db2jcc.jar;C:\db2\SQLLIB\java\sqlj.zip;C:\db2\SQLLIB\bin;C:\db2\SQLLIB\java\common.jar;C:\db2\SQLLIB\java\db2jcc_license_cisuz.jar;C:\db2\SQLLIB\java\db2jcc_license_cu.jar
    #    DB2 classpath automatically setup by DB2 install
    #
    try:
       log("SPMJython.initialize %s %s %s %s %s %s" %(dbType, dbServer, dbPort, dbAlias, dbName, dbUserUserName))
       spm_jython.initialize(dbType, dbServer, dbPort, dbAlias, dbName, dbUserUserName, dbUserPassword)
       log("SPMJython.initialize complete")
       backupdb(response_properties, "current.ddl")
    except Exception,e:
       log("ERROR: Cannot connect to the database, therefore can not backup tables.  Continuing.  (" +  e.getMessage() + ")");
       return "FAILURE"
       
    return "SUCCESS"

#
# Backup the data in the database.
#
def backupdb(response_properties, filename):
   #
   # Run At:
   # Read the REQUESTERS, SERVICES, OPERATIONS, POLICYTYPES, POLICIES, CONFIGPROPERTIES, CFGPROPERTIES tables, and make a backup of the data in a .DDL file.
   #
   
   logs_path = response_properties.get("LOG_PATH")
   if logs_path == None:
      log("ERROR: LOG_PATH variable is not defined.")
   
   ddl_path = logs_path + File.separator + filename
   log("Backup Database: %s" %(ddl_path))
   printWriter = None

   try:   
      printWriter = PrintWriter(FileOutputStream(File(ddl_path)))
   except Exception,e:
      e.printStackTrace()
      
   doTable("CONFIGPROPERTIES", printWriter)
   doTable("CFGPROPERTIES", printWriter)
   doTable("REQUESTERS", printWriter)
   doTable("SERVICES", printWriter)
   doTable("OPERATIONS", printWriter)
   doTable("POLICYTYPES", printWriter)
   doTable("POLICIES", printWriter)

   if printWriter != None:
      printWriter.close()   
      
   return
   
#
# Write table data to the printWriter.
# This does not stop on an error
# param: tableName the name of the database table
# param: printWriter where to write the data
#
def doTable(tableName, printWriter):
   log("Processing: " + tableName)
   try:
      spm_jython.backupTable(tableName, printWriter)
   except Exception, e:
      log(e.getMessage())
      # e.printStackTrace()   # debug only
   return
   
#
# Build the Full DDL path, and verify
# return "FAILURE" or full path
#
def buildDDLPath(ddl_root, ddl_name, ddl_type):
    # get ddl path
    ddl_path = ddl_root + os.sep + ddl_name + ddl_type + ".ddl"
    log("ddl_path: " + ddl_path)
    if os.path.exists(ddl_path) != 1:
       log("ERROR: DDL file does not exist.")
       return "FAILURE"
    return ddl_path
    

