#   begin_generated_IBM_prolog
#
#   prolog.pro.samp.telcosp
#   ******************************************************************
#   IBM Sample Source Materials
#
#   Sample source materials are supplied As-Is.
#   No warranty is expressed or implied.
#
#   Product(s): 5724-O05 soa
#
#   (C)Copyright IBM Corp. 2006, 2008
#
#   The source code for this program supplied under the terms of the
#   End User License Agreement (EULA) that accompanied this product.
#   ******************************************************************
#
#   end_generated_IBM_prolog

import sys.argv
import getopt
from com.ibm.twss.spm.admin.common import ScopedPolicy
from com.ibm.twss.spm.admin.common import PolicyValue
from java.util import Properties

from pytwss.mbean import spm_init, mbean_utils
from pytwss.utils import _, log

AC_SERVICE = 'http://www.csapi.org/wsdl/parlayx/payment/amount_charging/v2_1/interface'
RAC_SERVICE = 'http://www.csapi.org/wsdl/parlayx/payment/reserve_amount_charging/v2_1/interface'
VC_SERVICE = 'http://www.csapi.org/wsdl/parlayx/payment/volume_charging/v2_1/interface'
RVC_SERVICE = 'http://www.csapi.org/wsdl/parlayx/payment/reserve_volume_charging/v2_2/interface'
SVC_GROUP = 'Payment'
SERVICE_IMPL = 'PX21_PMT_POSTPAID'
AC_IMPL = 'PX21_PMT_POSTPAID_AC'
RAC_IMPL = 'PX21_PMT_POSTPAID_RAC'
VC_IMPL = 'PX21_PMT_POSTPAID_VC'
RVC_IMPL = 'PX21_PMT_POSTPAID_RVC'

def configurePolicies(response_properties, deploy_info):
	req_scope = 'ALL'
	svc_scope = SVC_GROUP
	op_scope = 'ALL'

	log(_("SOAC7520I") %{ '1' : SERVICE_IMPL })

	svc_admin = spm_init.ServiceAdministration()
	policy_admin = spm_init.PolicyAdministration()

	spm_init.registerServiceImplementation(
		svc_admin, AC_SERVICE, AC_IMPL, 1,
		'Parlay X 2.1 Payment Amount Charging Postpaid Service')
	spm_init.registerServiceImplementation(
		svc_admin, RAC_SERVICE, RAC_IMPL, 1,
		'Parlay X 2.1 Payment Reserve Amount Charging Postpaid Service')
	spm_init.registerServiceImplementation(
		svc_admin, VC_SERVICE, VC_IMPL, 1,
		'Parlay X 2.1 Payment Volume Charging Postpaid Service')
	spm_init.registerServiceImplementation(
		svc_admin, RVC_SERVICE, RVC_IMPL, 1,
		'Parlay X 2.1 Payment Reserve Volume Charging Postpaid Service')

        sp_root_url = response_properties.get("ServicePlatformRootURL")

	policies = [
		ScopedPolicy(
			requesterIdentifier=req_scope,
			serviceIdentifier=svc_scope,
			operation=op_scope,
			name='service.config.PaymentUsageRecordEnabled',
			value=PolicyValue(
				value='true',
				type='Boolean'
			)
		),
		ScopedPolicy(
			requesterIdentifier=req_scope,
			serviceIdentifier=svc_scope,
			operation=op_scope,
			name='service.config.ResultsUsageRecordEnabled',
			value=PolicyValue(
				value='false',
				type='Boolean'
			)
		),
		ScopedPolicy(
			requesterIdentifier=req_scope,
			serviceIdentifier=svc_scope,
			operation=op_scope,
			name='service.config.CEIEnabled',
			value=PolicyValue(
				value='false',
				type='Boolean'
			)
		),
		#ScopedPolicy(
		#	requesterIdentifier=req_scope,
		#	serviceIdentifier=svc_scope,
		#	operation=op_scope,
		#	name='service.config.Currency',
		#	value=PolicyValue(
		#		value='<someval>',
		#		type='String'
		#	)
		#),
		ScopedPolicy(
			requesterIdentifier=req_scope,
			serviceIdentifier=AC_IMPL,
			operation=op_scope,
			name='service.Endpoint',
			value=PolicyValue(
				value=sp_root_url + "/TWSS/ParlayX21/Payment/PostPaid/services/AmountCharging",
				type='String'
			),
		),
		ScopedPolicy(
			requesterIdentifier=req_scope,
			serviceIdentifier=VC_IMPL,
			operation=op_scope,
			name='service.Endpoint',
			value=PolicyValue(
				value=sp_root_url + "/TWSS/ParlayX21/Payment/PostPaid/services/VolumeCharging",
				type='String'
			) 
		),
		ScopedPolicy(
			requesterIdentifier=req_scope,
			serviceIdentifier=RAC_IMPL,
			operation=op_scope,
			name='service.Endpoint',
			value=PolicyValue(
				value=sp_root_url + "/TWSS/ParlayX21/Payment/PostPaid/services/ReserveAmountCharging",
				type='String'
			) 
		),
		ScopedPolicy(
			requesterIdentifier=req_scope,
			serviceIdentifier=RVC_IMPL,
			operation=op_scope,
			name='service.Endpoint',
			value=PolicyValue(
				value=sp_root_url + "/TWSS/ParlayX21/Payment/PostPaid/services/ReserveVolumeCharging",
				type='String'
			) 
		),
	]
	spm_init.createPolicies(policy_admin, policies)
	log(_("SOAC7521I") %{ '1' : SERVICE_IMPL })
        
        return "SUCCESS"

if __name__ == 'main':
   if len(sys.argv) > 0:
      sp_path = sys.argv[0]
   else:
      sp_path = "http://localhost:9080/"
   p = Properties()
   p.put("ServicePlatformRootURL", sp_path)
   configurePolicies(p, None)
