#   begin_generated_IBM_prolog
#
#   prolog.pro.samp.telcosp
#   ******************************************************************
#   IBM Sample Source Materials
#
#   Sample source materials are supplied As-Is.
#   No warranty is expressed or implied.
#
#   Product(s): 5724-O05 soa
#
#   (C)Copyright IBM Corp. 2008
#
#   The source code for this program supplied under the terms of the
#   End User License Agreement (EULA) that accompanied this product.
#   ******************************************************************
#
#   end_generated_IBM_prolog

import time, os, sys, os.path, shutil
from java.lang import System, Runtime, String, Long, Float, Integer, Exception
from java.util import Properties
from java.io import File, FileInputStream
from pytwss.utils import _, setLogFile, log, logScreen
import initdb, px21_pmt_postpaid_pol
import wsadmin_utils_cfg_app, wsadmin_utils_cfg

SERVICE_NAME = "PX21_PMT_POSTPAID"

#
# Perform the configuration process.
# param: java.util.Properties response_properties object with the defined properties
# param: deploy_info object used to query the server or cluster members.
# return: the result "SUCCESS" "FAILURE"
#
def configure(response_properties, deploy_info, selectedFeatures_properties):
	
    # Initialize variables
    service_marker_path = response_properties.getProperty("SERVICE_MARKER_PATH")
    ddl_root = response_properties.get("DDL_SERVICES_PATH")
    ddl_type = response_properties.get("DDL_TYPE")
    db_server_type = response_properties.get("DatabaseServerType")
    base_svc_path = response_properties.get("TWSS_SERVICES_PATH")
    #marker_path = service_marker_path + os.sep + SERVICE_NAME + ".properties"
    marker_path = selectedFeatures_properties.getProperty("Payment_PostPaid_over_Usage_Records")
        
    #if os.path.exists(marker_path) != 1:
    if marker_path == 'false':
       log("Skip Configuring: " + SERVICE_NAME)
       return "SUCCESS"
           	
    log("Configuring: " + SERVICE_NAME)
    logScreen(_("SOAC7533I") %{ '1' : SERVICE_NAME })
    
    # do initialization
    ddl_path = initdb.buildDDLPath(ddl_root, "AdminPmt", ddl_type)
    if ddl_path == "FAILURE":
       return "FAILURE"
    
    # initialize database
    result = initdb.executeDDL(response_properties, ddl_path)        
    if result != "SUCCESS":
       return "FAILURE"
      
    px21_pmt_postpaid_pol.configurePolicies(response_properties, deploy_info)

    # Get the Cluster or Cell id.
    scope = deploy_info.getClusterId()
    if scope == None or scope == "":
       scope = deploy_info.getCellId()
    log("Target scope: " + scope)
    
    # Get the Cluster or Server Id typically used for application deployment.
    dest_scope = deploy_info.getClusterId()
    if dest_scope == None or dest_scope == "":
       dest_scope = deploy_info.getServerIds()[0]
    log("Destination scope: " + dest_scope)
    
    # Get the service name and path.
    svc_path = base_svc_path + os.sep + 'PX21_PMT_POSTPAID.ear'      
    svc_name = wsadmin_utils_cfg_app.getAppName(svc_path)
    
    # Deploy the service.
    twss_alias = 'TWSSAlias'
    wsadmin_utils_cfg_app.installApp(svc_name, svc_path, dest_scope, db_type=db_server_type, alias=twss_alias)

    # Change the Classloader policy
    app_id = wsadmin_utils_cfg.getContainedCfgIdFromName(svc_name, 'Deployment')
    wsadmin_utils_cfg_app.modifyClassloaderPolicy(app_id, 'PARENT_FIRST')

    # Map the security roles to users and groups.
    twss_adm_group = response_properties.getProperty('TWSSAdminGroupName')
    wsadmin_utils_cfg_app.mapRole(svc_name, 'TWSSAdministrator', groups=[twss_adm_group])
    wsadmin_utils_cfg_app.mapRole(svc_name, 'PMT_POSTPAID_ServiceAccess ', all_auth="true")
        
    return "SUCCESS"

#
# Perform the syncServers process
# param: java.util.Properties response_properties object with the defined properties 
# return: the result "SUCCESS" "FAILURE"
#
def syncServers(response_properties, deploy_info, selectedFeatures_properties):
	
    # check component marker
    service_marker_path = response_properties.getProperty("SERVICE_MARKER_PATH")
        
    #marker_path = service_marker_path + os.sep + SERVICE_NAME + ".properties"    
    marker_path = selectedFeatures_properties.getProperty("Payment_PostPaid_over_Usage_Records")
        
    #if os.path.exists(marker_path) != 1:
    if marker_path == 'false':
       log("Skip SyncServers: " + SERVICE_NAME)
       return "SUCCESS"
           	
    log("SyncServers: " + SERVICE_NAME)

    # do initialization
    logScreen(_("SOAC7533I") %{ '1' : SERVICE_NAME })
    # more later
    
    return "SUCCESS"