#   begin_generated_IBM_prolog
#
#   prolog.pro.samp.telcosp
#   ******************************************************************
#   IBM Sample Source Materials
#
#   Sample source materials are supplied As-Is.
#   No warranty is expressed or implied.
#
#   Product(s): 5724-O05 soa
#
#   (C)Copyright IBM Corp. 2006, 2008
#
#   The source code for this program supplied under the terms of the
#   End User License Agreement (EULA) that accompanied this product.
#   ******************************************************************
#
#   end_generated_IBM_prolog

import sys.argv
import getopt
from com.ibm.twss.spm.admin.common import ScopedPolicy
from com.ibm.twss.spm.admin.common import PolicyValue
from java.util import Properties

from pytwss.mbean import spm_init, mbean_utils
from pytwss.utils import _, log

SD_SERVICE = 'http://www.ibm.com/wsdl/wappush/send/v1_0/interface'
NI_SERVICE = 'http://www.ibm.com/wsdl/wappush/notification/v1_0/interface'
SVC_GROUP = 'WAP Push'
SERVICE_IMPL = 'WAP10_PUSH_SMPP'
SD_IMPL = 'WAP10_PUSH_SMPP_SD'
NI_IMPL = 'WAP10_PUSH_SMPP_NI'

def configurePolicies(response_properties, deploy_info):
	req_scope = 'ALL'
	svc_scope = SVC_GROUP
	op_scope = 'ALL'

	log(_("SOAC7520I") %{ '1' : SERVICE_IMPL })

	svc_admin = spm_init.ServiceAdministration()
	policy_admin = spm_init.PolicyAdministration()

	spm_init.registerServiceImplementation(
		svc_admin, SD_SERVICE, SD_IMPL, 1,
		'WAPPush Messaging Send SMPP Service')
	spm_init.registerServiceImplementation(
		svc_admin, NI_SERVICE, NI_IMPL, 1,
		'WAPPush Messaging Notification SMPP Service')

        sp_root_url = response_properties.get("ServicePlatformRootURL")

	policies = [
		ScopedPolicy(
			requesterIdentifier=req_scope,
			serviceIdentifier=svc_scope,
			operation=op_scope,
			name='service.standard.ChargingSupported',
			value=PolicyValue(
				value='true',
				type='Boolean'
			)
		),
		ScopedPolicy(
			requesterIdentifier=req_scope,
			serviceIdentifier=svc_scope,
			operation=op_scope,
			name='service.standard.MaximumTargets',
			value=PolicyValue(
				value='5',
				type='Numeric'
			)
		),
		ScopedPolicy(
			requesterIdentifier=req_scope,
			serviceIdentifier=svc_scope,
			operation=op_scope,
			name='service.config.CallLevelPrivacy',
			value=PolicyValue(
				value='false',
				type='Boolean'
			)
		),
		ScopedPolicy(
			requesterIdentifier=req_scope,
			serviceIdentifier=svc_scope,
			operation=op_scope,
			name='service.config.ProcessUpToLimit',
			value=PolicyValue(
				value='true',
				type='Boolean'
			)
		),
		ScopedPolicy(
			requesterIdentifier=req_scope,
			serviceIdentifier=svc_scope,
			operation=op_scope,
			name='service.config.MessagePriority',
			value=PolicyValue(
				value='0',
				type='String'
			)
		),
		ScopedPolicy(
			requesterIdentifier=req_scope,
			serviceIdentifier=svc_scope,
			operation=op_scope,
			name='service.config.ConfirmDelivery',
			value=PolicyValue(
				value='false',
				type='Boolean'
			)
		),
		ScopedPolicy(
			requesterIdentifier=req_scope,
			serviceIdentifier=svc_scope,
			operation=op_scope,
			name='service.config.StatusRetainTime',
			value=PolicyValue(
				value='1440', # Minutes in 24 hours
				type='Numeric'
			)
		),
		ScopedPolicy(
			requesterIdentifier=req_scope,
			serviceIdentifier=svc_scope,
			operation=op_scope,
			name='service.config.WAPPushMessageValidTime',
			value=PolicyValue(
				value='43200', # Minutes in 30 days
				type='Numeric'
			)
		),
		ScopedPolicy(
			requesterIdentifier=req_scope,
			serviceIdentifier=svc_scope,
			operation=op_scope,
			name='service.config.WAPPushAction',
			value=PolicyValue(
				value='Signal-medium',
				type='String'
			)
		),
		ScopedPolicy(
			requesterIdentifier=req_scope,
			serviceIdentifier=svc_scope,
			operation=op_scope,
			name='service.config.useWAPPushActionFromUser',
			value=PolicyValue(
				value='false',
				type='Boolean'
			)
		),
		ScopedPolicy(
			requesterIdentifier=req_scope,
			serviceIdentifier=SD_IMPL,
			operation=op_scope,
			name='service.Endpoint',
			value=PolicyValue(
				value=sp_root_url + "/TWSS/WAP10/WAPPush/SMPP/services/SendWAPPush",
				type='String'
			),
		),
		#ScopedPolicy(
		#	requesterIdentifier=req_scope,
		#	serviceIdentifier=NI_IMPL,
		#	operation=op_scope,
		#	name='service.Endpoint',
		#	value=PolicyValue(
		#		value=sp_root_url + "/TWSS/WAP10/WAPPush/SMPP/services/WAPPushNotification",
		#		type='String'
		#	),
	]
	spm_init.createPolicies(policy_admin, policies)
	log(_("SOAC7521I") %{ '1' : SERVICE_IMPL })
        
        return "SUCCESS"

if __name__ == 'main':
   if len(sys.argv) > 0:
      sp_path = sys.argv[0]
   else:
      sp_path = "http://localhost:9080/"
   p = Properties()
   p.put("ServicePlatformRootURL", sp_path)
   configurePolicies(p, None)
