import os
from pytwss.utils import log
import wsadmin_utils_cfg
import wsadmin_utils_cfg_db
import wsadmin_utils_cfg_jms
import wsadmin_utils_cfg_app

# The wsadmin references are not automatically available in
# imported scripts.  The main script, denoted by the wsadmin -f
# argument, should call setAdminRefs on the module before using
# the function.
def setAdminRefs(adminObjects):
	global AdminConfig, AdminControl, AdminTask, AdminApp
	(AdminConfig, AdminControl, AdminTask, AdminApp)  = adminObjects
        wsadmin_utils_cfg.setAdminRefs(adminObjects)
        wsadmin_utils_cfg_db.setAdminRefs(adminObjects)
        wsadmin_utils_cfg_jms.setAdminRefs(adminObjects)
        wsadmin_utils_cfg_app.setAdminRefs(adminObjects)
	
# Returns multiple line wsadmin list content into a python list.
# param:  inStr The wsadmin list content
# return: a python list representation of the wsadmin output
def wsadminToList(inStr):
        outList=[]
        if (len(inStr)>0 and inStr[0]=='[' and inStr[-1]==']'):
                tmpList = inStr[1:-1].split() #splits space-separated lists,
        else:
                tmpList = inStr.split("\n")   #splits for Windows or Linux
        for item in tmpList:
                item = item.rstrip();         #removes any Windows "\r"
                if (len(item)>0):
                        outList.append(item)
        return outList
#enddef

# Returns multiple line wsadmin list content into a python list.
# param:  inStr The wsadmin list content
# return: a python list representation of the wsadmin output
def wsadminToList2(inStr):
        outList=[]
        if (len(inStr)>0 and inStr[0]=='[' and inStr[-1]==']'):
                tmpList = inStr[1:-1].split() #splits space-separated lists,
        else:
                tmpList = inStr.split("\n")   #splits for Windows or Linux
        for item in tmpList:
                item = item.rstrip();         #removes any Windows "\r"
                outList.append(item)
        return outList
#enddef

# Searches a list of wsadmin ids for an id with
# a 'name'.
# param: lst The list of wsadmin ids to search.
# param: name The name of the target id.
# return: A list of ids that match the target name, usually of len 1.
def searchIdsByName(lst, name):
   idsFound = []
   for configId in lst:
      configName = getName(configId)
      if(configName == name):
         idsFound.append(configId)
      #endif
   #endfor
   return idsFound
#enddef

# Searches a list of wsadmin ids for an id with
# a 'name'.
# param: lst The list of wsadmin ids to search.
# param: name The name of the target id.
# param: attr The attribute to be searched for the name.
# return: A list of ids that match the target name, usually of len 1.
def searchIdsByAttrName(lst, name, attr):
   idsFound = []
   for configId in lst:
      configName = AdminConfig.showAttribute(configId, attr)
      if(configName == name):
      	idsFound.append(configId)
      #endif
   #endfor
   return idsFound
#enddef

# Parses a wsadmin id for the 'name' of the configuration object that
# the id represents.
# param: cfgId The name of the id to parse.
# return: The 'name' of the id.
def getName (cfgId):
    startIndex = 0
    if cfgId.startswith('"'):
       startIndex = 1
    endIndex = (cfgId.find("(") - 1)
    return cfgId[startIndex:endIndex+1]
#endDef 

# Gets the fully qualified host name of a node.
# param: node_id The wsadmin id of the node.
# returns the fully qualified host name of the node.
def getFullyQualifiedHostName(node_id):
    return AdminConfig.showAttribute(node_id, 'hostName')

def getShortHostName(node_id):
    host = AdminConfig.showAttribute(node_id, 'hostName')
    return host.split(".")[0]
    
def getServerNameFromId(scope):
    outputStr = None
    stIndex = scope.find("(") + 1
    endIndex = scope.find("|")   
    scopeLst = scope[stIndex:endIndex].split("/")
    scopeListLen = len(scopeLst)
    for i in range(0, scopeListLen, 2):
       scopeType = scopeLst[i]
       scopeValue = scopeLst[i+1]
       if(scopeType == "servers"):
          outputStr = scopeValue
    return outputStr
    
def getNodeNameFromId(scope):
    outputStr = None
    stIndex = scope.find("(") + 1
    endIndex = scope.find("|")   
    scopeLst = scope[stIndex:endIndex].split("/")
    scopeListLen = len(scopeLst)
    for i in range(0, scopeListLen, 2):
       scopeType = scopeLst[i]
       scopeValue = scopeLst[i+1]
       if(scopeType == "nodes"):
          outputStr = scopeValue
    return outputStr

def getCellNameFromId(scope):
    outputStr = None
    stIndex = scope.find("(") + 1
    endIndex = scope.find("|")   
    scopeLst = scope[stIndex:endIndex].split("/")
    scopeListLen = len(scopeLst)
    for i in range(0, scopeListLen, 2):
       scopeType = scopeLst[i]
       scopeValue = scopeLst[i+1]
       if(scopeType == "cells"):
          outputStr = scopeValue
    return outputStr
    
def getClusterNameFromId(scope):
    outputStr = None
    stIndex = scope.find("(") + 1
    endIndex = scope.find("|")   
    scopeLst = scope[stIndex:endIndex].split("/")
    scopeListLen = len(scopeLst)
    for i in range(0, scopeListLen, 2):
       scopeType = scopeLst[i]
       scopeValue = scopeLst[i+1]
       if(scopeType == "clusters"):
          outputStr = scopeValue
    return outputStr

    
#
# Get the name of this Id
#
def getNameFromId(scope):
    endIndex = scope.find("(")
    name = scope[0:endIndex]
    return name

#
# Get the cell id from the cell name.
# param: cell_name the cell name
# return: the cell id
#
def getCellId(cell_name):
    cell_id = AdminConfig.getid("/Cell:%s/" % cell_name)
    if len(cell_id.splitlines()) != 1:
       errMsg = 'Expected one cell id, got %s' % len(cell_id)
       raise RuntimeError, errMsg
    return cell_id

#
# Returns "true if the scope is a cell id.
#
def isCellId(scope):
    outputStr = None
    stIndex = scope.find("(") + 1
    endIndex = scope.find("|")   
    scopeLst = scope[stIndex:endIndex].split("/")
    scopeListLen = len(scopeLst)
    if scopeListLen == 2:
       return "true"
    else:
       return "false"
       	
#
# Get the cluster id from the cluster name.
# param: cluster_name the cluster name
# return: the cluster id
#
def getClusterId(cluster_name):
    cluster_id = AdminConfig.getid("/ServerCluster:%s/" % (cluster_name))
    if len(cluster_id.splitlines()) != 1:
       errMsg = 'Expected one cluster id, got %s' % len(cluster_id)
       raise RuntimeError, errMsg
    return cluster_id

#
# Get the server id from the node and server names.
# param: node_name the node name
# param: server_name the server name
# return: the cell id
#
def getServerId(node_name, server_name):
    server_id = AdminConfig.getid("/Node:%s/Server:%s/" % (node_name, server_name))
    if len(server_id.splitlines()) != 1:
       errMsg = 'Expected one server id, got %s' % len(server_id)
       raise RuntimeError, errMsg
    return server_id
    
#
# Get the server id from the node and server names.
# param: node_name the node name
# param: server_name the server name
# return: the cell id
#
def hasNodeAgent(node_name):
    server_id = AdminConfig.getid("/Node:%s/Server:nodeagent/" % (node_name))
    if server_id == None or len(server_id) == 0:
       return "false"
    return "true"

#
# Get the node id from the node name.
# param: the node name
# return: the node id
#
def getNodeId(node_name):
    node_id = AdminConfig.getid("/Node:%s/" % (node_name))
    if len(node_id.splitlines()) != 1:
       errMsg = 'Expected one node id, got %s' % len(node_id)
       raise RuntimeError, errMsg
    return node_id

#
# Get the cell name.
# return: the cell name
#
def getCellName():
    return AdminControl.getCell()
    
def validateClstrOrSrvr(scope):
	
    cluster_name = getClusterNameFromId(scope)
    server_name = getServerNameFromId(scope)
    node_name = getNodeNameFromId(scope)
    	
    if not cluster_name:
       if(not (server_name and node_name)):
          raise RuntimeError, "The scope must be a server id or a cluster id."

def buildScope(scope):
    outputStr = None
    stIndex = scope.find("(") + 1
    endIndex = scope.find("|")   
    scopeLst = scope[stIndex:endIndex].split("/")
    scopeListLen = len(scopeLst)
    for i in range(0, scopeListLen, 2):
       scopeType = scopeLst[i]
       scopeValue = scopeLst[i+1]
       if(scopeType == "cells"):
          outputStr = "/Cell:%s/" % scopeValue
       if(scopeType == "clusters"):
          outputStr += "ServerCluster:%s/" % scopeValue
       if(scopeType == "nodes"):
          outputStr += "Node:%s/" % scopeValue
       if(scopeType == "servers"):
          outputStr += "Server:%s/" % scopeValue
    return outputStr
    
def buildScope2(cell=None, node=None, server=None, cluster=None, app=None):
    outputStr = None
    if node:
       outputStr = 'Node=' + node
       if server:
          outputStr = outputStr + ',Server=' + server  
    elif cluster:
       outputStr = 'Cluster=' + cluster
    elif app:
       outputStr = 'Application=' + app
    elif cell:
       outputStr = 'Cell=' + cell
    return outputStr

# Replaces occurences of \ with the os separator
# param: a path with \'s
# return: the correctly formatted path according to the os
def getOsPath(path):
    return path.replace("\\", os.sep)
    
