from pytwss.utils import log, _, logScreen
from java.lang import System, Runtime, String, Long, Float, Integer, Exception
import wsadmin_utils
from pytwss.mbean import mbean_utils
import time, sys

# The wsadmin references are not automatically available in
# imported scripts.  The main script, denoted by the wsadmin -f
# argument, should call setAdminRefs on the module before using
# the function.
def setAdminRefs(adminObjects):
    global AdminConfig, AdminControl, AdminTask, AdminApp
    (AdminConfig, AdminControl, AdminTask, AdminApp)  = adminObjects

# Enables Java 2 Security
# param: scope The scope in which to enable Security.
def enableJava2Security(scope):
    log("wsadmin_utils_cfg enableJava2Security (%s)" % (scope))
    security = getSecurity()
    attrs = [['enforceJava2Security', 'true']]
    modifyAttrs(security, attrs)

# Disables Java 2 Security
# param: scope The scope in which to disable Security. 
def disableJava2Security(scope):
    log("wsadmin_utils_cfg disableJava2Security (%s)" % (scope))
    security = getSecurity()
    attrs = [['enforceJava2Security', 'false']]
    modifyAttrs(security, attrs)

# Returns true if the wsadmin proces is a dm_environment.
# return: true if the wsadmin proces is a dm_environment.
def isFederated():
    log("wsadmin_utils_cfg isFederated")
    is_federated = AdminTask.isFederated()
    return is_federated
    
# Creates a Jvm Property at the desired scope if a Property
# with the same name does not exist.
# param: name The name of the JVM system property.
# param: value The value of the JVM system property.
# param: scope The scope of the property.
# return: the wsadmin id of the created or existing Jvm Property.
def createJvmProperty(name, value, scope):
    log("wsadmin_utils_cfg createJvmProperty (%s, %s, %s)" % (name, value, scope))
    parent_id = getJvm(scope)
    return createProperty(name, value, parent_id, 'systemProperties')

# Creates a HAManagerService Property at the desired scope if a Property
# with the same name does not exist.
# param: name The name of the HAManagerService property.
# param: value The value of the HAManagerService property.
# param: scope The scope of the property.
# return: the wsadmin id of the created or existing HAManagerService Property.
def createHAProperty(name, value, scope):
    log("wsadmin_utils_cfg createHAProperty (%s, %s, %s)" % (name, value, scope))
    parent_id = getHAManagerService(scope)
    return createProperty(name, value, parent_id, 'properties')

# Creates a Web Container Property at the desired scope if a Property
# with the same name does not exist.
# param: name The name of the Web Container property.
# param: value The value of the Web Container property.
# param: scope The scope of the property.
# return: the wsadmin id of the created or existing Web Container Property.
def createWebContainerProperty(name, value, scope):
    log("wsadmin_utils_cfg createWebContainerProperty (%s, %s, %s)" % (name, value, scope))
    parent_id = getWebContainer(scope)
    return createProperty(name, value, parent_id, 'properties')
	
# Creates a SIP Container Property at the desired scope if a Property
# with the same name does not exist.
# param: name The name of the SIP Container property.
# param: value The value of the SIP Container property.
# param: scope The scope of the property.
# return: the wsadmin id of the created or existing SIP Container Property.
def createSipContainerProperty(name, value, scope):
    log("wsadmin_utils_cfg createSipContainerProperty (%s, %s, %s)" % (name, value, scope))
    parent_id = getSIPContainer(scope)
    return createProperty(name, value, parent_id, 'properties')

# Creates a Property at the desired scope if a Property with the same
# name does not exist.
# param: name The name of the Property.
# param: value The value of the Property.
# param: scope The scope of the property.
# return: the wsadmin id of the created or existing Property.
def createProperty(name, value, parent_id, attr):
    log("wsadmin_utils_cfg createProperty (%s, %s, %s, %s)" % (name, value, parent_id, attr))
    attrs = [["name", name], ["value", value]]
    cfg_id = getCfgIdFromParentAttrLst(name, parent_id, attr)
    if(not cfg_id):
       cfg_id = createConfig("Property", parent_id, attrs, attr)
    else:
       log("Reusing existing Configuration (%s)." % cfg_id)       
    return cfg_id

# Note: The mediationThreadPool is defined by ESB Server, we don't create it ourself, but set some of its properties.    
# Creates a Thread Pool for a configuration if a Thread Pool with the same
# name does not exist.
# param: me_id The id of the parent configuration in which to create the pool.
# param: min_size The min size of the ThreadPool
# param: max_size The max size of the ThreadPool
# param: inactivity_timeout The inactivity timeout of the ThreadPool
# return: the wsadmin id of the created or existing Thread Pool property.
def createMediationThreadPool(me_id, min_size, max_size, inactivity_timeout):
    log("wsadmin_utils_cfg createMediationThreadPool (%s, %s, %s, %s)" % (me_id, min_size, max_size, inactivity_timeout))
    tp_attrs = [['minimumSize', min_size],['maximumSize', max_size],['inactivityTimeout', inactivity_timeout]]	
    cfg_id = getCfgIdFromParentAttrLst("ThreadPool", me_id, "mediationThreadPool")
    if(not cfg_id):
       cfg_id = createConfig("ThreadPool", me_id, tp_attrs, "mediationThreadPool")
    else:
       log("Reusing existing Configuration (%s)." % cfg_id)       
    return cfg_id
    
# Creates a Thread Pool for a configuration if a Thread Pool with the same
# name does not exist.
# param: server_id The id of the parent configuration in which to create the pool.
# param: name the name of the pool
# param: min_size The min size of the ThreadPool
# param: max_size The max size of the ThreadPool
# param: inactivity_timeout The inactivity timeout of the ThreadPool
# return: the wsadmin id of the created or existing Thread Pool property.
def createServerThreadPool(server_id, name, min_size, max_size, inactivity_timeout):
    log("wsadmin_utils_cfg createServerThreadPool (%s, %s, %s, %s, %s)" % (server_id, name, min_size, max_size, inactivity_timeout))
    tp_attrs = [['minimumSize', min_size],['maximumSize', max_size],['inactivityTimeout', inactivity_timeout],['name',name]]	
    tpm = getThreadPoolManager(server_id)
    log(tpm)
    tpool = getThreadPool(name, server_id)
    if(not tpool):
       tpool = createConfig("ThreadPool", tpm, tp_attrs)
       log("created %s" %(tpool))
    else:
       log("Reusing existing Configuration (%s)." % tpool)       
    return tpool

# Note: Sets a set of properties on a DataSource.  (Question: does this same technique work on other non-DataSource J2EE Resources like JMS, RA, etc?)
# Creates a J2EEResourcePropertySet in a datasource.  This function does not check
# to see if the property set already exists.
# param: ds_id The wsadmin id of a datsource.
# param: attrs The attributes of the J2EEResourcePropertySet, typically []
# return: the wsadmin id of the J2EEResourcePropertySet.
def createJ2EEResourcePropertySet(ds_id, attrs):
    log("wsadmin_utils_cfg createJ2EEResourcePropertySet (%s, %s)" % (ds_id, attrs))
    cfg_id = createConfig("J2EEResourcePropertySet", ds_id, attrs)
    return cfg_id

# Creates a J2EEResourceProperty in a J2EEResourcePropertySet if a property
# with the same name does not exist.
# param: prop_set The wsadmin id of a J2EEResourcePropertySet
# param: name The name of the J2EEResourceProperty.
# param: value The value of the J2EEResourceProperty.
# param: prop_type The type of the J2EEResourceProperty.
# return: the wsadmin id of the J2EEResourceProperty.
def createJ2EEResourceProperty(prop_set, name, value, prop_type):
    log("wsadmin_utils_cfg createJ2EEResourceProperty (%s, %s, %s, %s)" % (prop_set, name, value, prop_type))
    attrs = [["name", name], ["type", prop_type], ["value", value]]
    cfg_id = getCfgIdFromParentAttrLst(name, prop_set, "resourceProperties")
    if(not cfg_id):
       cfg_id = createConfig("J2EEResourceProperty", prop_set, attrs, "resourceProperties")
    else:
       log("Reusing existing Configuration (%s)." % cfg_id)
    return cfg_id

# Creates a JAASAuthData if a JAASAuthData with the same alias does not
# exist.
# param: alias The alias id of a JAASAuthData
# param: user_id The user_id of the JAASAuthData.
# param: password The password of the JAASAuthData.
# param: description The description of the JAASAuthData.
# param: scope The scope of the JAASAuthData.
# return: the wsadmin id of the JAASAuthData.
def createJAASAuthData(alias, user_id, password, description, scope):
    log("wsadmin_utils_cfg createJAASAuthData (%s, %s, %s, %s)" % (alias, user_id, description, scope))
    security = getSecurity()
    attrs = [['alias', alias], ['userId', user_id], ['password', password], ['description', description]]
    cfg_id = getCfgIdFromAttr(alias, "JAASAuthData", "alias")
    if cfg_id:
       log("Removing existing Configuration (%s)." % cfg_id)
       AdminConfig.remove(cfg_id)
    cfg_id = createConfig("JAASAuthData", security, attrs, "authDataEntries")
    return cfg_id

# Creates a StringNameSpaceBinding if a StringNameSpaceBinding with 
# the same name does not exist.
#
# param: ns_id The name.
# param: ns_name The nameInNameSpace.
# param: bind_str The stringToBind.
# param: scope The scope of the StringNameSpaceBinding.
# return: the wsadmin id of the StringNameSpaceBinding.
def createStringNameSpaceBinding(ns_id, ns_name, bind_str, scope):
    log("wsadmin_utils_cfg createStringNameSpaceBinding (%s, %s, %s, %s)" % (ns_id, ns_name, bind_str, scope))
    attrs = [['name', ns_id], \
             ['nameInNameSpace', ns_name], \
             ['stringToBind', bind_str]]
    cfg_id = getCfgIdFromName(ns_id, "StringNameSpaceBinding", scope)
    if(not cfg_id):
       cfg_id = createConfig("StringNameSpaceBinding", scope, attrs)
    else:
       log("Reusing existing Configuration (%s)." % cfg_id)
    return cfg_id

# Creates a SchedulerConfiguration at a given scope using
# the default work manager and a table prefix of SCHED.
#
# parm: name The unique name of the SchedulerConfiguration.
# parm: jndi_name The jndiName
# parm: ds_jndi_name The datasourceJNDIName
# parm: poll_interval The pollInterval
# parm: alias The datasourceAlias
# parm: scope The scope.
# return: the wsadmin id of the created or existing SchedulerConfiguration
def createScheduler(name, jndi_name, ds_jndi_name, poll_interval, alias, scope):
    log("wsadmin_utils_cfg createScheduler (%s, %s, %s, %s, %s, %s)" \
       % (name, jndi_name, ds_jndi_name, poll_interval, alias, scope))
    table_prefix = 'SCHED'
    wm = 'wm/default'
    use_admin_roles = 'true'
    attrs = [['name', name], \
             ['datasourceJNDIName', ds_jndi_name], \
             ['datasourceAlias', alias], \
             ['jndiName', jndi_name], \
             ['pollInterval', poll_interval], \
             ['useAdminRoles', use_admin_roles], \
             ['tablePrefix', table_prefix], \
             ['workManagerInfoJNDIName', wm]]
    cfg_id = getCfgIdFromName(name, 'SchedulerConfiguration', scope)
    if not cfg_id:
       sched_prov = getSchedulerProvider(scope)
       cfg_id = createConfig("SchedulerConfiguration", sched_prov, attrs)
    else:
       log("Reusing existing Configuration (%s)." % cfg_id)
       
    return cfg_id

# Creates a Work Area Partition on a server.  If one exists with the same name,
# it will be reused.
#
# param: name The name of the WorkAreaPartition
# param: max_rcv_size The maxReceiveSize.
# param: max_send_size the maxSendSize.
# param: enable_at_start "true" or "false"
# param: bidirectional "true" or "false"
# param: def_attr_ser "true" or "false"
# param: enable_ws_prop "true" or "false"
# param: server_id The wsadmin id of a server.
# return: the wsadmin id of the created or existing WorkAreaPartition
def createWorkAreaPartition(name, max_rcv_size, max_send_size, enable_at_start, bidirectional, def_attr_ser, enable_ws_prop, server_id):
    log("wsadmin_utils_cfg createWorkAreaPartition (%s, %s, %s, %s, %s, %s, %s, %s)" \
       % (name, max_rcv_size, max_send_size, enable_at_start, bidirectional, def_attr_ser, enable_ws_prop, server_id))
    attrs = [['name', name], \
             ['description', name], \
             ['maxReceiveSize', max_rcv_size], \
             ['maxSendSize', max_send_size], \
             ['bidirectional', bidirectional], \
             ['enable', enable_at_start], \
             ['enableWebServicePropagation', enable_ws_prop], \
             ['deferredAttributeSerialization', def_attr_ser]]
    waps = getWorkAreaPartitionService(server_id)
    cfg_id = getCfgIdFromParentAttrLst(name, waps, "partitions")
    if(not cfg_id):
       cfg_id = createConfig('WorkAreaPartition', waps, attrs, 'partitions')
    else:
       log("Reusing existing WorkAreaPartition (%s)." % cfg_id)
       

def createConfig(cfg_type, parent_id, attrs, attr=None):
    if(attr):
       newid = AdminConfig.create(cfg_type, parent_id, attrs, attr)
    else:
       newid = AdminConfig.create(cfg_type, parent_id, attrs)
    return newid
    
# Gets an environment variable at a given scope.
# param: name The name of the environment variable.
# param: scope The scope of the environment variable.
def getEnvironmentVariable(name, scope):
    log("wsadmin_utils_cfg getEnvironmentVariable (%s, %s)" % (name, scope))
    # The scope for this method is in scope=value,[scope=value] form.
    # The buildScope2 function can be used to help build the scope in
    # the appropriate format.    
    cluster_name = wsadmin_utils.getClusterNameFromId(scope)
    server_name = wsadmin_utils.getServerNameFromId(scope)
    node_name = wsadmin_utils.getNodeNameFromId(scope)
    cell_name = wsadmin_utils.getCellNameFromId(scope)      
    cvrted_scope = wsadmin_utils.buildScope2(cell=cell_name, node=node_name, server=server_name, cluster=cluster_name)
    inputStr = '[-scope %s -variableName %s]' % (cvrted_scope, name) 
    return AdminTask.showVariables(inputStr)
      
# Sets an environment variable at a given scope.
# param: name The name of the environment variable.
# param: value The value of the environment variable.
# param: description The desc of the environment variable.
# param: scope The scope of the environment variable.
def setEnvironmentVariable(name, value, description, scope):
    log("wsadmin_utils_cfg setEnvironmentVariable (%s, %s, %s, %s)" % (name, value, description, scope))
    # The scope for this method is in scope=value,[scope=value] form.
    # The buildScope2 function can be used to help build the scope in
    # the appropriate format.    
    cluster_name = wsadmin_utils.getClusterNameFromId(scope)
    server_name = wsadmin_utils.getServerNameFromId(scope)
    node_name = wsadmin_utils.getNodeNameFromId(scope)
    cell_name = wsadmin_utils.getCellNameFromId(scope)      
    cvrted_scope = wsadmin_utils.buildScope2(cell=cell_name, node=node_name, server=server_name, cluster=cluster_name)
    inputStr = '[-scope %s  -variableName %s -variableValue %s -variableDescription %s]' % (cvrted_scope, name, value, description) 
    AdminTask.setVariable(inputStr)
    
# Sets an environment variable at a node scope.
# param: name The name of the environment variable.
# param: value The value of the environment variable.
# param: description The desc of the environment variable.
# param: scope The scope of the environment variable.
def setEnvironmentVariableOnNode(name, value, description, scope):
    log("wsadmin_utils_cfg setEnvironmentVariable (%s, %s, %s, %s)" % (name, value, description, scope))
    # The scope for this method is in scope=value,[scope=value] form.
    # The buildScope2 function can be used to help build the scope in
    # the appropriate format.    
    server_name = wsadmin_utils.getServerNameFromId(scope)
    node_name = wsadmin_utils.getNodeNameFromId(scope)
    cvrted_scope = wsadmin_utils.buildScope2(cell=None, node=node_name, server=server_name, cluster=None)
    inputStr = '[-scope %s  -variableName %s -variableValue %s -variableDescription %s]' % (cvrted_scope, name, value, description) 
    AdminTask.setVariable(inputStr)

# Sets the Trace Settings for a given server.
# param: num_logs The rolloverSize.
# param: size The maxNumberOfBackupFiles
def setTraceLogSettings(num_logs, size, server_id):
    log("wsadmin_utils_cfg setTraceLogSettings (%s, %s, %s)" % (num_logs, size, server_id))
    trc = getTraceService(server_id)
    trc_log = getCfgIdFromParentAttr(trc, 'traceLog')
    attrs =  [['rolloverSize', num_logs], \
              ['maxNumberOfBackupFiles', size]]
    modifyAttrs(trc_log, attrs)
    	
# Sets the SystemOut Settings for a given server.
# param: num_logs The rolloverSize.
# param: size The maxNumberOfBackupFiles
def setSysOutLogSettings(num_logs, size, server_id):
    log("wsadmin_utils_cfg setSysOutLogSettings (%s, %s, %s)" % (num_logs, size, server_id))
    sysout = getSystemOut(server_id)
    attrs =  [['rolloverSize', num_logs], \
              ['maxNumberOfBackupFiles', size]]
    modifyAttrs(sysout, attrs)
    
# Sets the SystemErr Settings for a given server.
# param: num_logs The rolloverSize.
# param: size The maxNumberOfBackupFiles
def setSysErrLogSettings(num_logs, size, server_id):
    log("wsadmin_utils_cfg setSysErrLogSettings (%s, %s, %s)" % (num_logs, size, server_id))
    syserr = getSystemErr(server_id)
    attrs =  [['rolloverSize', num_logs], \
              ['maxNumberOfBackupFiles', size]]
    modifyAttrs(syserr, attrs)
    
# Sets the WC Default host and port for a given server.
# param: server_id The wsadmin id of the server.
# param: port The port of the server.
def setWCDefaultHostPort(server_id, port):
    log("wsadmin_utils_cfg setWCDefaultHostPort (%s, %s)" % (server_id, port))
    server_name = wsadmin_utils.getServerNameFromId(server_id)
    node_name = wsadmin_utils.getNodeNameFromId(server_id)
    if not(server_name and node_name):
    	raise RuntimeError, 'The server id is not valid.'
    node_id = wsadmin_utils.getNodeId(node_name)
    host_name = wsadmin_utils.getFullyQualifiedHostName(node_id)
    AdminTask.modifyServerPort (server_name, '[-nodeName %s -endPointName WC_defaulthost -host %s -port %s -modifyShared true]' % (node_name, host_name, port))

def setEJBContainerDataSourceSettings(jndi_name, scope):
    ejb_container = getEJBContainer(scope)
    attrs =  [['defaultDatasourceJNDIName', jndi_name]]
    AdminConfig.modify(ejb_container, attrs)
	
def createHostAlias(hostname, port):
    log("wsadmin_utils_cfg createHostAlias (%s, %s)" % (hostname, port))
    attrs = [["hostname", hostname], ["port", port]]
    parent_id = getCfgIdFromName('default_host', 'VirtualHost')
    host_aliases = wsadmin_utils.wsadminToList(AdminConfig.showAttribute(parent_id, 'aliases'))
    cfg_id = None
    for alias in host_aliases:
       pres_host = AdminConfig.showAttribute(alias, 'hostname')
       pres_port = AdminConfig.showAttribute(alias, 'port')
       if hostname == pres_host and port == pres_port:
          cfg_id = alias
          break
    if not cfg_id:
       cfg_id = createConfig('HostAlias', parent_id, attrs, 'aliases')
    else:
       log("Reusing existing Configuration (%s)." % cfg_id)  
    return cfg_id


def setEJBTimerDataSourceSettings(jndi_name, alias, scope):
    ejb_time = getEJBTimer(scope)
    attrs =  [['datasourceJNDIName', jndi_name],['datasourceAlias', alias]]
    AdminConfig.modify(ejb_time, attrs)

def setEJBTimerThreadSettings(num_alarm_threads, scope):
    ejb_time = getEJBTimer(scope)
    attrs =  [['numAlarmThreads', num_alarm_threads]]
    AdminConfig.modify(ejb_time, attrs)

def setJvmHeapSize(initial_size, max_size, scope):
    log("wsadmin_utils_cfg setJvmHeapSize (%s, %s, %s)" % (initial_size, max_size, scope))
    attrs = [['initialHeapSize', initial_size],['maximumHeapSize', max_size]]	
    modifyAttrs(getJvm(scope), attrs)
    
def setJvmGcc(gcc, scope):
    log("wsadmin_utils_cfg setJvmGcc (%s, %s)" % (gcc, scope))
    attrs = [['genericJvmArguments', gcc]]	
    modifyAttrs(getJvm(scope), attrs)

def setHATransportBfrSz(size, scope):
    log("wsadmin_utils_cfg setHATransportBfrSz (%s, %s)" % (size, scope))
    attrs = [['transportBufferSize', size]]	
    modifyAttrs(getHAManagerService(scope), attrs)

def setDefaultThreadPoolSize(min_size, max_size, server_id):
    log("wsadmin_utils_cfg setDefaultThreadPoolSize (%s, %s, %s)" % (min_size, max_size, server_id))	
    attrs = [['minimumSize', min_size],['maximumSize', max_size]]
    tpool = getThreadPool("Default", server_id)
    modifyAttrs(tpool, attrs)
    
def setServerThreadPoolSize(name, min_size, max_size, server_id):
    log("wsadmin_utils_cfg setServerThreadPoolSize (%s, %s, %s, %s)" % (name, min_size, max_size, server_id))	
    attrs = [['minimumSize', min_size],['maximumSize', max_size]]
    tpool = getThreadPool(name, server_id)
    modifyAttrs(tpool, attrs)
    
def setWebContainerThreadPoolSize(min_size, max_size, server_id):
    log("wsadmin_utils_cfg setWebContainerThreadPoolSize (%s, %s, %s)" % (min_size, max_size, server_id))	
    attrs = [['minimumSize', min_size],['maximumSize', max_size]]	
    tpool = getThreadPool("WebContainer", server_id)
    modifyAttrs(tpool, attrs)

# I'm not sure if we configure through the Thread Manager or ORB Service.
# Need to converse.
# Use the ORB.thread.pool settings associated with the Thread Pool Manager 
def setORBThreadPoolSize(min_size, max_size, server_id):
    log("wsadmin_utils_cfg setORBThreadPoolSize (%s, %s, %s)" % (min_size, max_size, server_id))	
    attrs = [['minimumSize', min_size],['maximumSize', max_size]]	
    tpool = getThreadPool("ORB.thread.pool", server_id)
    modifyAttrs(tpool, attrs)

# I'm not sure if we configure through the Thread Manager or ORB Service.
# Need to converse.
# Use the Thread Pool Settings directly associated with the ORB service. 
def setORBServiceThreadPoolSize(min_size, max_size, server_id):
    log("wsadmin_utils_cfg setORBServiceThreadPoolSize (%s, %s, %s)" % (min_size, max_size, server_id))	
    attrs = [['minimumSize', min_size],['maximumSize', max_size]]
    orb = getObjectRequestBroker(server_id)
    orb_pool = getCfgIdFromParentAttr(orb, "threadPool")
    modifyAttrs(orb_pool, attrs)
    
def setMediationThreadPoolSize(min_size, max_size, server_id):
    log("wsadmin_utils_cfg setMediationThreadPoolSize (%s, %s, %s)" % (min_size, max_size, server_id))	
    attrs = [['minimumSize', min_size],['maximumSize', max_size]]	
    tpool = getThreadPool("mediationThreadPool", server_id)
    if tpool != None:                           # note this pool may or may note be there, if unit testing on WAS
       modifyAttrs(tpool, attrs)
    else:
       log("WARNING: mediationThreadPool was not found, can not set its size. continuing...")
    
# Assigns a thread pool to the SIP Container.
# param: pool_id The thread pool id assigned to the SIP Container.
# param: server_id The server id of the SIP Container
def setSIPContainerThreadPool(pool_id, server_id):
    modifyAttrs(getSIPContainer(server_id), [['threadPool', pool_id]])

# Gets a Server Port (host/port) of a port type.
# param: port_type The type of port, i.e 'BOOSTRAP_ADDRESS'
# param: server_id The wsadmin id of the server.
# return: a tuple containing (host, port)
def getServerPort(port_type, server_id):
    log("wsadmin_utils_cfg getServerPort (%s, %s)" % (port_type, server_id))
    server_name = wsadmin_utils.getServerNameFromId(server_id)
    if not server_name:
    	raise RuntimeError, 'The server id is not valid.'
    node_name = wsadmin_utils.getNodeNameFromId(server_id)
    node_id = AdminConfig.getid('/Node:%s/' % (node_name))
    portTuple = None
    serverEntries = wsadmin_utils.wsadminToList(AdminConfig.list('ServerEntry', node_id))
    for serverEntry in serverEntries:
       sName = AdminConfig.showAttribute(serverEntry, "serverName")
       if sName == server_name:
          sepString = AdminConfig.showAttribute(serverEntry, "specialEndpoints")
          sepList = sepString[1:len(sepString)-1].split(" ")
          for specialEndPoint in sepList:
             endPointNm = AdminConfig.showAttribute(specialEndPoint, "endPointName")
             if endPointNm == port_type:
                ePoint = AdminConfig.showAttribute(specialEndPoint, "endPoint")
                hostname = AdminConfig.showAttribute(ePoint, 'host')
                portname = AdminConfig.showAttribute(ePoint, 'port')
                portTuple = (hostname, portname)
                break
    return portTuple
	
# Gets the ThreadPool id for a given thread pool name.
# param: name The name of the ThreadPool
# param: server_id The server_id of the ThreadPool
# return: the ThreadPool id.
def getThreadPool(name, server_id):
    tpm = getThreadPoolManager(server_id)
    return getCfgIdFromParentAttrLst(name, tpm, 'threadPools')

# Gets the ThreadPoolManager id.
# param: scope The server_id of the ThreadPoolManager
# return: the ThreadPoolManager id.
def getThreadPoolManager(server_id):
    return getCfgId('ThreadPoolManager', server_id)
    
# Gets the DefaultCoreGroup id.
# return: the DefaultCoreGroup id.
def getDefaultCoreGroup():
    groupName = AdminTask.getDefaultCoreGroupName()
    return AdminConfig.getid("/CoreGroup:"+groupName+"/" )
     
# Gets the JavaVirtualMachine id for a given scope.
# param: scope The scope of the JavaVirtualMachine
# return: the JavaVirtualMachine id of a scope.
def getJvm(scope):
    return getCfgId('JavaVirtualMachine', scope)

# Gets the ObjectRequestBroker id for a given scope.
# param: server_id The server id of the ObjectRequestBroker
# return: the wsadmin id of the ObjectRequestBroker.
def getObjectRequestBroker(server_id):
    return getCfgId('ObjectRequestBroker', server_id)
    
# Gets the HAManagerService id for a server.
# param: server_id The server id of the HAManagerService
# return: the HAManagerService id.
def getHAManagerService(server_id):
    return getCfgId('HAManagerService', server_id)

# Gets the TraceService id for a server.
# param: server_id The server id of the TraceService
# return: the TraceService id.
def getTraceService(server_id):
    return getCfgId('TraceService', server_id)

# Gets the SystemOut id for a server.
# param: server_id The server id of the SystemOut
# return: the SystemOut id.
def getSystemOut(server_id):
    return getCfgIdFromParentAttr(server_id, 'outputStreamRedirect')

# Gets the SystemErr id for a server.
# param: server_id The server id of the SystemErr
# return: the SystemErr id.
def getSystemErr(server_id):
    return getCfgIdFromParentAttr(server_id, 'errorStreamRedirect')
    
# Gets the SchedulerService id for a server.
# param: server_id The server id of the SchedulerService
# return: the SchedulerService id.
def getSchedulerService(server_id):
    return getCfgId('SchedulerService', server_id)

# Gets a Jvm Property at the desired scope.
# param: name The name of the JVM system property.
# param: value The value of the JVM system property.
# param: scope The wsadmin id of the server or node agent in which
# the property will be set.
# return: the wsadmin id of the existing JVM property.
def getJvmProperty(name, value, scope):
    jvm = AdminConfig.list('JavaVirtualMachine', scope)
    attrs = [["name", name], ["value", value]]
    ids = wsadmin_utils.wsadminToList(AdminConfig.showAttribute(jvm, "systemProperties"))
    propIds = wsadmin_utils.searchIdsByName(ids, name)
    if(len(propIds) != 0):
       return propIds[0]
    return None
	
# Gets the WebContainer id for a given scope.
# param: scope The scope of the JavaVirtualMachine
# return: the WebContainer id of a scope.
def getWebContainer(scope):
    return getCfgId('WebContainer', scope)
    
# Gets the SIPContainer id for a given scope.
# param: scope The scope of the SIPContainer
# return: the SIPContainer id of a scope.
def getSIPContainer(scope):
    return getCfgId('SIPContainer', scope)
    
    
# Gets the WorkAreaPartitionService id for a server.
# param: server_id The server_id of the WorkAreaPartitionService
# return: the WorkAreaPartitionService id.
def getWorkAreaPartitionService(server_id):
    return getCfgId('WorkAreaPartitionService', server_id)

# Gets the EJBTimer id for a given scope.
# param: scope The scope of the EJBTimer
# return: the EJBTimer id of a scope.
def getEJBTimer(scope):
    return getCfgId('EJBTimer', scope)

# Gets the EJBContainer id for a given scope.
# param: scope The scope of the EJBContainer
# return: the EJBContainer id of a scope.
def getEJBContainer(scope):
    return getCfgId('EJBContainer', scope)
      
# Gets the Security id for a given scope.
# return: the Security id of a scope.
def getSecurity():
    return getCfgId('Security')

# Gets the SchedulerProvider id for a scope.
# param: server_id The server_id of the SchedulerProvider
# return: the SchedulerProvider id.
def getSchedulerProvider(scope):
    return getContainedCfgId('SchedulerProvider', scope)
    
# Filters the AdminConfig.list so that configuration objects are only returned
# that are specifically at the cell scope.
def adm_list(cfg_type, scope):
    lst = wsadmin_utils.wsadminToList(AdminConfig.list(cfg_type, scope)) 

    if wsadmin_utils.isCellId(scope) == "true":
       newlst = []
       for scoped_id in lst:
          if wsadmin_utils.isCellId(scoped_id) == "true":
             log("CELL SCOPE %s" % scoped_id)
             newlst.append(scoped_id)   
          else:
             log("NOT CELL SCOPE")
       lst = newlst
    return lst
       
# Filters config id based on the scope and type.  This method
# should only be used when the scope and type alone is enough
# to filter to one id.
# param: type The type of the config object.
# param: scope The scope of the config object.
# return: the Config id of a scope.
def getCfgId(cfg_type, scope=None):	
    final_id = None
    if scope:
       cfg = adm_list(cfg_type, scope)
    else:
       cfg = wsadmin_utils.wsadminToList(AdminConfig.list(cfg_type))

    num_ids = len(cfg)
    if num_ids > 1:
    	raise RuntimeError, "Received %s %s configuration ids (%s)." % (num_ids, cfg_type, cfg)
    elif num_ids == 1:
    	final_id = cfg[0]
    return final_id
                
    	     
# Filters config id based on the scope, type, and the name
# in front of the configuration id.
# param: name The name of the config object.
# param: type The type of the config object.
# param: scope The scope of the config object.
# return: the Config id of a scope.
def getCfgIdFromName(name, cfg_type, scope=None):	
    final_id = None
    if scope:
       cfgList = adm_list(cfg_type, scope)
    else:
       cfgList = wsadmin_utils.wsadminToList(AdminConfig.list(cfg_type))
    cfg = wsadmin_utils.searchIdsByName(cfgList, name)
    num_ids = len(cfg)
    if num_ids > 1:
    	raise RuntimeError, "Received %s %s configuration ids (%s)." % (num_ids, cfg_type, cfg)
    elif num_ids == 1:
    	final_id = cfg[0]
    return final_id              	
	
# Filters config id based on the scope, type, and the attribute
# that holds the configuration's name.
# param: name The name of the config object.
# param: type The type of the config object.
# param: attr The attr that holds the name.  Usually is 'name', but
# could be something like 'alias'
# param: scope The scope of the config object.
# return: the Config id.
def getCfgIdFromAttr(name, cfg_type, attr, scope=None):
    final_id = None
    if scope:
       cfgList = adm_list(cfg_type, scope)
    else:
       cfgList = wsadmin_utils.wsadminToList(AdminConfig.list(cfg_type))
    cfg = wsadmin_utils.searchIdsByAttrName(cfgList, name, attr)
    num_ids = len(cfg)
    if num_ids > 1:
    	raise RuntimeError, "Received %s %s configuration ids (%s)." % (num_ids, cfg_type, cfg)
    elif num_ids == 1:
    	final_id = cfg[0]
    return final_id
    
# Filters the config id in a context relative to the parent.
# param: parent_id The id of the parent that holds the attr containing the target config id.
# param: parent_attr The attribute of the parent_id that holds the target config id.
# return: the Config id.
def getCfgIdFromParentAttr(parent_id, parent_attr):
    cfg_id = AdminConfig.showAttribute(parent_id, parent_attr)
    return cfg_id

# Filters the config id in a context relative to the parent when the attribute is
# a list.
# param: parent_id The id of the parent that holds the attr containing the target config id.
# param: parent_attr The attribute of the parent_id that holds the target config id.
# return: the Config id.
def getCfgIdFromParentAttrLst(name, parent_id, parent_attr):
    final_id = None
    cfg_ids = wsadmin_utils.wsadminToList(AdminConfig.showAttribute(parent_id, parent_attr))
    cfg_id = wsadmin_utils.searchIdsByName(cfg_ids, name)
    num_ids = len(cfg_id)
    if num_ids > 1:
    	raise RuntimeError, "Received %s %s configuration ids (%s)." % (num_ids, cfg_type, cfg_id)
    elif num_ids == 1:
    	final_id = cfg_id[0]
    return final_id


# Filters config id based on the scope and type.  This method
# should only be used when the scope and type alone is enough
# to filter to one id.
# param: type The type of the config object.
# param: scope The scope of the config object.
# return: the Config id of a scope.
def getContainedCfgId(cfg_type, scope):	
    final_id = None
    convertedScope = wsadmin_utils.buildScope(scope) + "%s:/" % (cfg_type)
    cfg = wsadmin_utils.wsadminToList(AdminConfig.getid(convertedScope))
    num_ids = len(cfg)
    if num_ids > 1:
    	raise RuntimeError, "Received %s %s configuration ids (%s)." % (num_ids, cfg_type, cfg)
    elif num_ids == 1:
    	final_id = cfg[0]
    return final_id

                
# Filters config id based on the scope, type, and the name
# in front of the configuration id.
# param: name The name of the config object.
# param: type The type of the config object.
# param: scope The scope of the config object.
# return: the Config id of a scope.
def getContainedCfgIdFromName(name, cfg_type, scope=None):	
    final_id = None
    if scope:
       convertedScope = wsadmin_utils.buildScope(scope) + "%s:%s/" % (cfg_type, name)
    else:
       convertedScope = "/%s:%s/" % (cfg_type, name)
    cfgList = wsadmin_utils.wsadminToList(AdminConfig.getid(convertedScope))
    cfg = wsadmin_utils.searchIdsByName(cfgList, name)
    num_ids = len(cfg)
    if num_ids > 1:
    	raise RuntimeError, "Received %s %s configuration ids (%s)." % (num_ids, cfg_type, cfg)
    elif num_ids == 1:
    	final_id = cfg[0]
    return final_id
        
# Filters config id based on the scope, type, and the attribute
# that holds the configuration's name.
# param: name The name of the config object.
# param: type The type of the config object.
# param: attr The attr that holds the name.  Usually is 'name', but
# could be something like 'alias'
# param: scope The scope of the config object.
# return: the Config id.
def getContainedCfgIdFromAttr(name, cfg_type, attr, scope):
    final_id = None
    convertedScope = wsadmin_utils.buildScope(scope) + "%s:%s/" % (cfg_type, name)
    cfgList = wsadmin_utils.wsadminToList(AdminConfig.getid(convertedScope))
    cfg = wsadmin_utils.searchIdsByAttrName(cfgList, name, attr)
    num_ids = len(cfg)
    if num_ids > 1:
    	raise RuntimeError, "Received %s %s configuration ids (%s)." % (num_ids, cfg_type, cfg)
    elif num_ids == 1:
    	final_id = cfg[0]
    return final_id

# Modifies the attributes of a wsadmin Configuration object.
# param: name The name of the config object.
# param: attrs A list of attributes to modify within the config object.
def modifyAttrs(cfgId, attrs):
    log("Modifying %s with attributes %s." % (cfgId, attrs))
    for i in range(10):
       try:
          AdminConfig.modify(cfgId, attrs)
          return
       except:
          log("retrying modifyAttrs")
          continue;
    AdminConfig.modify(cfgId, attrs)

# Calls AdminConfig.save() to save the configuration. 	
def save():
    log("wsadmin_utils save")
    AdminConfig.save()
    log("wsadmin_utils save done")


#Enables the ApplicationProfileService on a given server.
# sid - the server id
def enableAppProfileSvc(sid):
    aps = AdminConfig.list('ApplicationProfileService', sid)
    attrs = [['enable', 'true']]
    AdminConfig.modify(aps, attrs)

#Disables the ApplicationProfileService on a given server.
# sid - the server id
def disableAppProfileSvc(sid):
    aps = AdminConfig.list('ApplicationProfileService', sid)
    attrs = [['enable', 'false']]
    AdminConfig.modify(aps, attrs)