from pytwss.utils import log, _
import wsadmin_utils
import wsadmin_utils_cfg

# The wsadmin references are not automatically available in
# imported scripts.  The main script, denoted by the wsadmin -f
# argument, should call setAdminRefs on the module before using
# the function.
def setAdminRefs(adminObjects):
    global AdminConfig, AdminControl, AdminTask, AdminApp
    (AdminConfig, AdminControl, AdminTask, AdminApp)  = adminObjects

# Creates a JDBC Provider.
# 
# param: provider_type The provider type (DB2|Oracle)
# param: path The driver classpath path
# param: native_path Not required for win/unix/linux.. only for zOS
# param: xa The XA (true|false)
# param: scope The scope of the JDBCProvider.
def createJDBCProvider ( provider_type, path, native_path, xa, scope ):
    log("wsadmin_utils_cfg_db createJDBCProvider (%s, %s, %s, %s, %s)" % (provider_type, path, native_path, xa, scope))
    
    # Determine properties based on provider_type
    if (provider_type == "DB2"):
       if (xa == "true"):
          provider_name = "DB2 Universal JDBC Driver Provider (XA)"
          impl_class_name = "com.ibm.db2.jcc.DB2XADataSource"
       else:
          provider_name = "DB2 Universal JDBC Driver Provider"
          impl_class_name = "com.ibm.db2.jcc.DB2ConnectionPoolDataSource"
    elif (provider_type == "Oracle"):
       if (xa == "true"):
          provider_name = "Oracle JDBC Driver (XA)"
          impl_class_name = "oracle.jdbc.xa.client.OracleXADataSource"
       else:
          provider_name = "Oracle JDBC Driver"
          impl_class_name = "oracle.jdbc.pool.OracleConnectionPoolDataSource"
    else:
        raise RuntimeError, "Provider Type must be DB2 or Oracle."
        
    provider = getJDBCProvider(provider_name, scope)
    
    if provider:
       log("Reusing existing Database provider " + provider_name)  
    else:
       attrs = [["classpath", path], \
                ["implementationClassName", impl_class_name], \
                ["name", provider_name], \
                ["providerType", provider_name], \
                ["description", provider_name], \
                ["xa", xa]]
       provider = AdminConfig.create("JDBCProvider", scope, attrs )
       log("Database provider (%s) created successfully." % provider_name)   
    return provider

# 
# Creates a new Datasource if one with the same name does not exist at the 
# specified  scope. Otherwise, return the existing Datasource. 
#
# param: db_type (DB2|Oracle)
# param: ds_name Datasource name
# param: jndi_name JNDI name
# param: stmt_cache_sz Statement Cache Size
# param: provider  provider
# param: auth_alias JAAS authentication alias name
# param: description description
# param: scope scope
# param: db_name db_name (for DB2, Cloudscape, and Derby)
# param: hostname hostname
# param: port port
def createGenericDatasource (db_type, ds_name, jndi_name, stmt_cache_sz, provider, auth_alias, description, db_name, scope, hostname, port):
   if db_type == "DB2":
      return createDB2Datasource(ds_name, jndi_name, stmt_cache_sz, provider, auth_alias, description, db_name, scope, hostname, port)
   if db_type == "Oracle":
      return createOracleDatasource(ds_name, jndi_name, stmt_cache_sz, provider, auth_alias, description, db_name, scope, hostname, port)
   return None

# Creates a new DB2 Datasource if one with the same name does not exist at the 
# specified  scope. Otherwise, return the existing Datasource. 
#
# param: ds_name Datasource name
# param: jndi_name JNDI name
# param: stmt_cache_sz Statement Cache Size
# param: provider  provider
# param: auth_alias JAAS authentication alias name
# param: description description
# param: db_name db_name (for DB2, Cloudscape, and Derby)
# param: scope scope
# param: hostname hostname
# param: port port
def createDB2Datasource (ds_name, jndi_name, stmt_cache_sz, provider, auth_alias, description, db_name, scope, hostname, port):
    log("wsadmin_utils_cfg_db createDB2Datasource (%s, %s, %s, %s, %s, %s, %s, %s, %s, %s)" \
       % (ds_name, jndi_name, stmt_cache_sz, provider, auth_alias, description, db_name, scope, hostname, port))
    jcc_type = '4'
    datasourceHelperClassname = "com.ibm.websphere.rsadapter.DB2UniversalDataStoreHelper"
    provider_type = getJDBCProviderType(provider)
    
    ds_attrs = [["name", ds_name], \
                ["description", description], \
                ["jndiName", jndi_name], \
                ["statementCacheSize", stmt_cache_sz], \
                ["authDataAlias", auth_alias], \
                ["datasourceHelperClassname", datasourceHelperClassname], \
                ["xaRecoveryAuthAlias", auth_alias], \
                ["providerType", provider_type]]
    radapter =  getJ2CResourceAdapter('WebSphere Relational Resource Adapter', scope)
    datasource = getDataSource(jndi_name, scope)
    
    if datasource: 
       ds_type = getDataSourceType(datasource)
       ds_name_prop = getResourcePropertyValue("databaseName", datasource)   
       ds_server_prop = getResourcePropertyValue("serverName", datasource)   
       ds_port_prop = getResourcePropertyValue("portNumber", datasource)   
       # If a Datasource exists, but is of another provider type, delete it and
       # create a new one.
       if(ds_type != provider_type or ds_name_prop != db_name or ds_server_prop != hostname or ds_port_prop != port):
       	  log("Found and removing a datasource with a different provider_type or properties: %s" % datasource)
          AdminConfig.remove(datasource)
          datasource = None
       
          
    #Create datasource if it doesn't exist. 
    if not datasource:
       datasource = createDataSource(jndi_name, provider, ds_attrs, scope)
       
       #Create J2EEResourcePropertySet
       prop_set = wsadmin_utils_cfg.createJ2EEResourcePropertySet(datasource, [])
       wsadmin_utils_cfg.createJ2EEResourceProperty(prop_set, "databaseName", db_name, "java.lang.String")
       wsadmin_utils_cfg.createJ2EEResourceProperty(prop_set, "driverType", jcc_type, "java.lang.Integer")       
       wsadmin_utils_cfg.createJ2EEResourceProperty(prop_set, "serverName", hostname, "java.lang.String")       
       wsadmin_utils_cfg.createJ2EEResourceProperty(prop_set, "portNumber", port, "java.lang.Integer")       
                  
       #Create the connection pool object...
       cp_attrs = [["connectionTimeout", 1800], \
                   ["maxConnections", 80], \
                   ["minConnections", 50], \
                   ["reapTime", 180], \
                   ["unusedTimeout", 1800], \
                   ["agedTimeout", 0]]
       createConnectionPool( datasource, cp_attrs )

       #Create the connection factory
       cf_attrs = [["name", ds_name + "_CF"], \
                   ["authMechanismPreference", "BASIC_PASSWORD"], \
                   ["cmpDatasource", datasource]]
       createCMPConnectorFactory( radapter, cf_attrs )
    
       log("Datasource %s created successfully." % datasource)
    else:
       log("Reusing Datasource %s." % datasource)
    return datasource

# Create a new Oracle Database if one with the same name does not exist at the 
# specified scope. Otherwise, return the existing Datasource. 
#
# param: ds_name Datasource name
# param: jndi_name JNDI name
# param: stmt_cache_sz Statement Cache Size
# param: provider  provider
# param: auth_alias JAAS authentication alias name
# param: description description
# param: db_name (used for Oracle SID)
# param: scope scope
# param: hostname hostname
# param: port port
def createOracleDatasource ( ds_name, jndi_name, stmt_cache_sz, provider, auth_alias, description, db_name, scope, hostname, port ):
    log("wsadmin_utils_cfg_db createOracleDatasource (%s, %s, %s, %s, %s, %s, %s, %s, %s, %s)" \
       % (ds_name, jndi_name, stmt_cache_sz, provider, auth_alias, description, db_name, scope, hostname, port))   
       
    SID = db_name
    tnsstring = "jdbc:oracle:thin:@"+hostname+":"+port+":"+SID
    
    # set datasourceHelperClassname    "com.ibm.websphere.rsadapter.OracleDataStoreHelper"
    datasourceHelperClassname = "com.ibm.websphere.rsadapter.Oracle10gDataStoreHelper"
    cp = AdminConfig.showAttribute(provider, "classpath")
    if cp.find("ojdbc5.jar") != -1:
       datasourceHelperClassname = "com.ibm.websphere.rsadapter.Oracle11gDataStoreHelper"

    provider_type = getJDBCProviderType(provider)
    ds_attrs = [["name", ds_name], \
               ["description", description], \
               ["authDataAlias", auth_alias], \
               ["jndiName", jndi_name], \
               ["statementCacheSize", stmt_cache_sz], \
               ["datasourceHelperClassname", datasourceHelperClassname], \
               ["providerType", provider_type]]  
    radapter =  getJ2CResourceAdapter('WebSphere Relational Resource Adapter', scope)
    datasource = getDataSource(jndi_name, scope)    
      
    if datasource: 
       ds_type = getDataSourceType(datasource)
       ds_url_prop = getResourcePropertyValue("URL", datasource)   
       # If a Datasource exists, but is of another provider type, delete it and
       # create a new one.
       if(ds_type != provider_type or ds_url_prop != tnsstring):
       	  log("Found and removing a datasource with a different provider_type or properties: %s" % datasource)
          AdminConfig.remove(datasource)
          datasource = None
    	
    # Check if the DataSource already exists
    # If DataSource does not yet exists, create a new one
    if not datasource:
       datasource = createDataSource(jndi_name, provider, ds_attrs, scope)

       #Create the datasource properties...
       prop_set = wsadmin_utils_cfg.createJ2EEResourcePropertySet(datasource, [])
       wsadmin_utils_cfg.createJ2EEResourceProperty(prop_set, "URL", tnsstring, "java.lang.String")

       #Create the connection pool object...
       cp_attrs = [["connectionTimeout", 1800], \
                   ["maxConnections", 80], \
                   ["minConnections", 50], \
                   ["reapTime", 180], \
                   ["unusedTimeout", 1800], \
                   ["agedTimeout", 0]]
       createConnectionPool( datasource, cp_attrs )

       #Create the connection factory
       cf_attrs = [["name", ds_name + "_CF"], \
                   ["authMechanismPreference", "BASIC_PASSWORD"], \
                   ["cmpDatasource", datasource]]
       createCMPConnectorFactory( radapter, cf_attrs )

       log("Datasource %s created successfully." % datasource)
    else:
       log("Reusing Datasource %s." % datasource)
    return datasource

# Creates a DataSource if one does not exist.
# param: jndi_name The unique identifier of the DataSource.
# param: provider_id The wsadmin id of the JDBCProvider 
# param: attrs The attributes of the Datasource
# return: the wsadmin id of the created or existing CMP Connector Factory. 
def createDataSource(jndi_name, provider_id, attrs, scope):
    log("wsadmin_utils_cfg_db createDataSource (%s, %s, %s, %s)" % (jndi_name, provider_id, attrs, scope))
    datasource = getDataSource(jndi_name, scope)
    if(not datasource):
       cfg_id = wsadmin_utils_cfg.createConfig("DataSource", provider_id, attrs)
    else:
       log("Reusing existing Configuration (%s)." % cfg_id)
    return cfg_id

# Creates a CMP Connector Factory for a DataSource if one does not exist.
# param: ds_id The id of the DataSource
# param: attrs The attributes of the CMP Connector Factory
# return: the wsadmin id of the created or existing CMP Connector Factory.  
def createCMPConnectorFactory(ds_id, attrs):
    log("wsadmin_utils_cfg_db createCMPConnectorFactory (%s, %s)" % (ds_id, attrs))
    return wsadmin_utils_cfg.createConfig("CMPConnectorFactory", ds_id, attrs)

# Creates a Connection Pool for a DataSource if one does not exist.
# param: ds_id The id of the DataSource
# param: attrs The attributes of the ConnectionPool
# return: the wsadmin id of the created or existing Connection Pool.
def createConnectionPool(ds_id, attrs):
    log("wsadmin_utils_cfg_db createConnectionPool (%s, %s)" % (ds_id, attrs))
    return wsadmin_utils_cfg.createConfig("ConnectionPool", ds_id, attrs, "connectionPool")

# Gets the JDBCProvider id with a given name and scope.
# param: name The name of the JDBCProvider
# param: scope The scope of the JDBCProvider
# return: the JDBCProvider id of a scope.
def getJDBCProvider(name, scope):	
    return wsadmin_utils_cfg.getCfgIdFromName(name, 'JDBCProvider', scope)

# Gets the JDBCProvider type.
# param: prov_id the JDBCProvider id of a scope.
# return: the providerType of the JDBCProvider 
def getJDBCProviderType(prov_id):
    return wsadmin_utils_cfg.getCfgIdFromParentAttr(prov_id, 'providerType')
    
# Gets the DataSource id with a given jndi name and scope.
# param: name The jndi name of the DataSource
# param: scope The scope of the DataSource
# return: the DataSource id of a scope.
def getDataSource(name, scope):
    return wsadmin_utils_cfg.getCfgIdFromAttr(name, 'DataSource', "jndiName", scope)

# Gets the DataSource type.
# param: ds_id the DataSource id of a scope.
# return: the providerType of the datasource 
def getDataSourceType(ds_id):
    return wsadmin_utils_cfg.getCfgIdFromParentAttr(ds_id, 'providerType')
    
# Gets the J2CResourceAdapter id with a given name and scope.
# param: name The name of the J2CResourceAdapter
# param: scope The scope of the J2CResourceAdapter
# return: the J2CResourceAdapter id of a scope.
def getJ2CResourceAdapter(name, scope):
    return wsadmin_utils_cfg.getContainedCfgIdFromName(name, 'J2CResourceAdapter', scope)
    
# Gets the ConnectionPool id for a given data source name and scope.
# param: ds_id The id of the data source.
# param: scope The scope of the ConnectionPool
# return: the ConnectionPool id.
def getConnectionPool(ds_id):
    return wsadmin_utils_cfg.getCfgIdFromParentAttr(ds_id, 'connectionPool')
   
# Get Resource Property Value
# param: name of the property
# param: datasource id
def getResourcePropertyValue(name, datasource):
    log("getResourcePropertyName " + name)
    prop_set = wsadmin_utils_cfg.getCfgId("J2EEResourcePropertySet", datasource)
    if prop_set != None:
       prop_id = wsadmin_utils_cfg.getCfgIdFromParentAttrLst(name, prop_set, "resourceProperties")
       if prop_id != None:
          value = wsadmin_utils_cfg.getCfgIdFromParentAttr(prop_id, "value")
          log("  " + value)
          return value
    return None

